/*********************************************************************
 *
 *                Microchip C18 Firmware - PICBell18
 *
 *********************************************************************
;    Filename:	    Frequenz.c	(Frequenzimetro come da Kit LX1525 di N.E.) *
;    Date:          25/4/06                                          *
;    File Version:  1.00														              *
;                                                                     *
;    Author:        Dario Greggio                                     *
;    Company:       ADPM Synthesis sas                                *
 * Dependencies:    See INCLUDES section below
 * Processor:       PIC18
 * Compiler:        C18 2.30.01+
 * Company:         Microchip Technology, Inc.
 *
 * Author               Date        Comment
 *~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 * Dario Greggio        24/04/06    .
 ********************************************************************/

/** I N C L U D E S **********************************************************/
#include <p18cxxx.h>
#include <usart.h>
#include <portb.h>
#include <delays.h>
#include <timers.h>
#include <string.h>
#include <stdlib.h>
#include <stdio.h>
#include <reset.h>
#include <typedefs.h>


#include "frequenz.h"
#include "io_cfg.h"             // I/O pin mapping
#include "lcd.h"


#ifdef __18F1320

#pragma config WDT = ON, WDTPS = 128, MCLRE=OFF, STVR=ON, LVP=OFF
#pragma config OSC = HSPLL, FSCM=OFF, IESO=ON, PWRT=ON, BOR=ON, BORV=27

#endif




/** V A R I A B L E S ********************************************************/





#pragma udata
byte InitOK;			// idem...




extern byte LCDCols,LCDRows;	
extern byte LCDX,LCDY;	
extern byte LuceLCD,ContrLCD,StatoLCD;


extern byte kbKeys[MAX_TASTI_COMTEMPORANEI];
extern byte kbKeysOld[MAX_TASTI_COMTEMPORANEI];
extern byte kbScanCode;









byte EECheck;


#pragma	udata

long int frequency,frequencyToAdd;
word frequencyTempHigh;
word sampleDivider;
byte freqRange;



#pragma romdata
rom static const char FREQUENZ_C[]="$Id:FREQUENZ18.C Ver 1.0.1 - 27/7/13";
static const rom word SampleDividers[]={ 250,25 };
extern rom const char CopyrDate[];

// ---------------------------------------------------------------------




#pragma udata

extern byte FLAGS;        //GENERAL PURPOSE FLAG


/** P R I V A T E  P R O T O T Y P E S ***************************************/
void doMain(void);

void high_isr(void);
void low_isr(void);

void MandaTastoPremuto(byte);
void MandaTastoPremuto2(void);
void MandaMousePremuto(void);
void MandaTastoRilasciato(byte);

short int GetByte232(void);
void PutByte232(byte);
void sendQueryStatus232(void);


/******************************************************************************
 * Function:        void (void)
 *
 * PreCondition:    None
 *
 * Input:           None
 *
 * Output:          None
 *
 * Side Effects:    None
 *
 * Overview:        This function is a place holder for other user routines.
 *                  It is a mixture of both USB and non-USB tasks.
 *
 * Note:            None
 *****************************************************************************/


// METTERE un FLAG per disattivare scroll automatico (CHR$13) su ultima riga, per sfruttarla meglio??




//	__IDLOCS 0x4744

// QUARZO_40MHZ 
// 100nS/istruzione @ 10MHz








#ifdef USA_232

struct COMM_STATUS CommStatus;
byte Byte2Send;
byte ByteRec;
byte Buf232Ptr;					//contatore per scrittura buffer 232
byte Buf232PtrI;				//contatore per lettura buffer 232
byte Buf232[16];								// buffer 16 byte per ricezione I2C e 232 (BANCO 1)

#endif

char buffer[16];



//#ifdef __16F74				; I2C slave software, solo 74 ... NO! usati come temp in RX anche da 874!!
extern byte I2CData;
extern byte I2CCnt;
//#endif




#define CR          0xd
#define LF          0xa
#define BEL					0x7
#define BS					0x8
#define TAB					0x9
#define DC1					0x11
#define DC2					(DC1+1)
#define DC3					(DC1+2)
#define DC4					(DC1+3)
#define FF					0xc
#define CAN					0x18				// stesso di FF ossia CLS
#define ESC					0x1b
// VTAB, curs giu', curs dx??

#define FLASH_TIME  7





//
//***** VARIABLE DEFINITIONS

//
word Counter;
byte CounterL2;
byte Timer1;
extern byte FLAGS_ESC;					// per sequenze ESCAPE: 0=SetLuce, 1=SetContr, 2=SetFreq; 3=SetLed; 4&5=SetXY (1&2); 6=SetKClick
word Tmr1Base;


extern byte TimerBuzz;
extern byte BeepFreq;



byte Clock_S;			// orologio hh:mm:ss gg:mm:yyyy, mirrored in EEPROM
byte Clock_M;
byte Clock_H;
byte Clock_D;
byte Clock_m;
byte Clock_Y;
byte Clock_10;		// (decimi di secondo per dividere interrupt); ANCHE MARKER_GOOD_EEPROM (solo in Eprom, v.)





// Flag bits (FLAGS)
#define NOSCANK	0
#define NOKCLICK	1
#define CLOCK_ON		3
#define IN_ESCAPE_CHAR  4
#define BUZZON		5
#define TRIGM2	6
#define TRIGK		7
#define TRIGM		7





//**********************************************************************



//extern void _startup (void);        // See c018i.c in your C18 compiler dir
//#pragma code _RESET_INTERRUPT_VECTOR = 0x000800
//void _reset(void) {
//  _asm goto _startup _endasm
//	}

	/****************************************************************************
*
*  high_interrupt
*  Descrizione: gestore dell'interrupt
*  Riferimento:
*  Parametri in ingresso: nessuno
*  Parametri in uscita: nessuno
*  Note:
*
****************************************************************************/
#pragma code high_vector=0x08
void interrupt_at_high_vector(void) {
  _asm goto high_isr _endasm
	}
#pragma code

#pragma code low_vector=0x18
void interrupt_at_low_vector(void) {
  _asm goto low_isr _endasm
	}
#pragma code



/****************************************************************************
*
*  timer_isr
*  Descrizione: gestore dell'interrupt del Timer 0
*  Riferimento:
*  Parametri in ingresso: nessuno
*  Parametri in uscita: nessuno
*  Note:
*
****************************************************************************/
#pragma interruptlow low_isr nosave=section("MATH_DATA"),section(".tmpdata")
void low_isr(void) {

	if(PIE1bits.TMR1IE && PIR1bits.TMR1IF) {
		PIR1bits.TMR1IF = 0;
		Nop();
		Nop();
		Nop();	// 9 998 860
		Nop();
		Nop();
		Nop();	// 9 999 610 (@ x10),	10.000.436 (@ x1)
		// ca 250 x NOP
		Nop();

		WriteTimer1(Tmr1Base);			// re-inizializzo TMR1

//		m_Strobe1Bit ^= 1;


		if(!--sampleDivider) {
			sampleDivider=SampleDividers[freqRange];
//			frequency = frequencyTempHigh;
//			frequency <<= 16;
			frequency = (word)ReadTimer0();
			*((word *)(((char *)&frequency)+2)) = frequencyTempHigh;
			WriteTimer0(0);
			frequencyTempHigh=0;
			m_Led0Bit ^= 1;
			}


		Timer1++;				// 4mSec

#ifdef USA_KB_BUZZER
		if(!(Timer1 & 0x3)) {		//16mSec ca.
			if(FLAGS & (1 << BUZZON)) {				// OCCHIO alla contemporaneita' col BEEP (chr$7)
		//		CCPR1L=255;				// spengo PWM1 (faccio in modo che non si attivi mai... v. anche PR2)
		//	movlw  00000000b
		//	movwf  T2CON
				FLAGS &= ~(1 << BUZZON);
				}
			}
#endif


		if(!(Timer1 & 0x1f) && !(FLAGS & (1 << NOSCANK))) {		//125mSec ca.
		


	//				m_Led2Bit ^= 1;

			ClrWdt();
		
			if(!m_Puls0Bit)	{
				kbKeys[0]++;
				kbKeys[0] |= 1;			// questo indica la pressione (b0)
				kbKeys[0] &= 0x1f;	// questo indica la press. prolungata (b4)
//				checkKey(/*kbScanCode*/);
				}
			else
				kbKeys[0]=0;
			if(!m_Puls1Bit)	{
				kbKeys[1]++;
				kbKeys[1] |= 1;
				kbKeys[1] &= 0x1f;
				}
			else
				kbKeys[1]=0;
			if(!m_Puls2Bit)	{
				kbKeys[2]++;
				kbKeys[2] |= 1;
				kbKeys[2] &= 0x1f;
				}
			else
				kbKeys[2]=0;

		
			FLAGS |= (1 << TRIGK);
		
			}			// NOSCANK


			
		if(!(Timer1 & 0x3f)) {		//256mSec ca.

	// avendo CLRWDT nell'IRQ, in pratica non salta MAI!!

			Clock_10++;
			if(Clock_10 == 4) {			// FINIRE!!
				Clock_10=0;
				Clock_S++;
				if(Clock_S == 60) {
					Clock_S=0;
					Clock_M++;
					if(Clock_M == 60) {
						Clock_M=0;
						Clock_H++;
						if(Clock_M == 24) {
							Clock_H=0;
							}
						}
					}
				}

			}

		}		// PIR1IF...

	}

#pragma interrupt high_isr nosave=section("MATH_DATA"),section(".tmpdata")
void high_isr(void) {

	if(INTCONbits.TMR0IE && INTCONbits.TMR0IF) {
		INTCONbits.TMR0IF=0;
		frequencyTempHigh++;
		}

	if(/*PIR1bits.TXIF || */
		PIE1bits.RCIE && PIR1bits.RCIF)	{	// serve il controllo su RCIE, se abiliti e disabiliti IRQ da programma...

#ifdef USA_232
	//	bcf  PIR1,RCIF

		if(RCSTAbits.FERR)
			CommStatus.COMM_FRERR=1;
		if(RCSTAbits.OERR) {
			RCSTAbits.CREN=0;
			Nop(); Nop();
			RCSTAbits.CREN=1;								// forse basta BCF (v. datasheet), anche se non e' chiaro...
	//	bsf CommStatus,COMM_OVRERR				; serve l'overrun??
			}


	//	bsf CommStatus,COMM_OVL						; gestire overflow buffer?
		ByteRec=ReadUSART();					// QUESTO pulisce interrupt!
																	// salvo cmq anche qui...
		Buf232[Buf232Ptr++]=ByteRec;				// max 16
		Buf232Ptr &= 0xf;
#endif
	
		}

	}






/** D E C L A R A T I O N S **************************************************/
#pragma code
void main(void) {
	static short int ch;
	static byte i;
	overlay byte *p,*p2;
	overlay byte tempC,tempC2;			// temporanea, usata da KB ecc.

	// enable portb weak pull-up (e irq su fronte discesa) to detected change in PORTB; TMR0 con prescaler 1:32



/*========================================================================
;       RESET ENTRY
;========================================================================*/
start:
cold_reset:

	InitOK=RCON 	/*STATUS*/;
	StatusReset();

	RCONbits.IPEN=1;				// interrupt in Modalit Advanced
//	INTCONbits.PEIE = 1;		// attiva interrupt perif (SERVE x USART!) NON + v. low/high
	IPR1bits.TMR1IP = 0;			// Timer-1 Low Pty
	INTCON2bits.RBIP = 0;				// PortB (non usato) Low Pty
	IPR1bits.RCIP = 1;					// USART High Pty
//	INTCONbits.GIEL = 1;			// attiva interrupt low pty (STESSO BIT!)
//	INTCONbits.GIE = 1;			// attiva interrupt globali


	ClrWdt();					// DOPO!

	RCONbits.NOT_BOR=1;
	RCONbits.NOT_POR=1;




	OpenTimer0(TIMER_INT_ON & T0_16BIT & T0_SOURCE_EXT & T0_PS_1_1 /*& T0_EDGE_FALL*/);		// SEMPRE high pty!
	OpenTimer1(TIMER_INT_ON & T1_16BIT_RW & T1_SOURCE_INT & T1_OSC1EN_OFF & 
		/*T1_SOURCE_CCP & */ T1_PS_1_8 & T1_SYNC_EXT_OFF);

		// OKKIO che i 50MHz sull'ingresso del Timer0, si ottengono solo se prescaler  impostato in certo modo
		// v. Forum e tabella parametri del Timer 0



//	movlw	00000111b		; enable portb weak pull-up (e irq su fronte discesa); TMR0 con prescaler 1:256
//	movwf	OPTION_REG

	EnablePullups();

// SE si usa warm/cold reset, lo stato degli I/O va ripristinato dopo ogni reset!!!

	ADCON1=0x7F;	//01111111b;			// ??? nessun analogico (MA BISOGNA METTERE ST'ISTRUZIONE! non e' Power-On Default!)
//	CMCON=0x7;




	LATA=0b00000000;				// 
	LATB=0b00000000;


	TRISA=0b00111100;					// TimerInput, 1 tasto, 2 strobe, 1 led
	TRISB=0b00000001;					// output cmd LCD, 2 tasti



	WriteTimer0(0);					//clr tmr0 & prescaler
	WriteTimer1(0);					//clr tmr1

//	movlw	00000111b	; pull-ups B, tmr0 enable, prescaler Timer with 1:256 prescaler FINIRE!!
//	movwf	OPTION_REG

	INTCONbits.GIE=0;			// disabilito tutti interrupt
	INTCONbits.GIEL=0;			// disabilito tutti interrupt


	ClrWdt();

#ifdef USA_232

	OpenUSART(USART_RX_INT_ON & USART_TX_INT_OFF & USART_ASYNCH_MODE & 
		USART_EIGHT_BIT & USART_CONT_RX & USART_BRGH_HIGH,
		23);
//	INTCONbits.PEIE = 1;			// attiva interrupt perif (SERVE x USART!
	INTCONbits.GIEL = 1;			// attiva interrupt low pty (STESSO BIT!)
// v. skyusb18.c, e/oppure usare irq low/high
//	baudUSART(BAUD_16_BIT_RATE & BAUD_WAKEUP_OFF & BAUD_AUTO_OFF);
	// x= (Fosc / (Brate * 16 (BRGH=1) ) ) ) -1
	// Fosc=44236800 (11.0592 XTAL)
	
#endif

	Tmr1Base=TMR1BASE_100;
	
	WriteTimer1(Tmr1Base);					// inizializzo TMR1

	BeepFreq=BEEP_STD_FREQ;

// il timer 2  usato per il beep e il PWM...
//	CCP1CON=0b00001100;			//			 TMR2: PWM mode
	CCP1CON=0b00000000;			//			 NO: RB3 mi SERVE!
//	OpenPWM();
//	CCPR1L=255;
//	movlw		255
//	movwf	 CCPR1H				; questo  read-only in modalit PWM

//	T2CON=0x7;	//00000111b		// accendo il timer2, il che attiva entrambi i PWM; prescaler=16
//	T2CON=0b00001111;		// accendo il timer2, il che attiva entrambi i PWM; prescaler=16; postscaler=1:2


//	PR2=BEEP_STD_FREQ;

	

	ClrWdt();

init_i2c:
#ifdef I2C_MSSP_HARDWARE			// default su 874...
															// per ora uso solo SW, su 74... FINIRE (se funziona!)!

	SSPADD=(((40000000/100000)/4) -1);    // read selected baud rate (100KHz)
	//; initialize I2C baud rate
	SSPSTAT=0b00000000;	            // select I2C input levels; enable slew rate

#ifdef I2C_MSSP_HARDWARE	
	SSPCON2=0;
#endif

#ifdef __16F74								//
	SSPCON=0b00111011;							// Firmware Controlled Master Mode, Slave Idle
#else
	SSPCON=0b00111000;							// Master mode, SSP enable
// AN578 mette 00111110b...
//	SSPCON=0b00111110;							// Master mode, SSP enable
#endif

#endif

//	OpenI2C()







	if(InitOK & (1 << 3 /*NOT_TO*/))	 	// se era watchdog...
		{


		}





	LCDInit();				// ripristino display... (ANCHE dopo WARM_RESET?)




	if(EEleggi(&EECheck) == 0x47) {				// se  gi stata inizializzata, salto!
		EEcopiaARAM(&LuceLCD);
		EEcopiaARAM(&ContrLCD);

		EEcopiaARAM(&Clock_S);
		EEcopiaARAM(&Clock_M);
		EEcopiaARAM(&Clock_H);
		EEcopiaARAM(&Clock_D);
		EEcopiaARAM(&Clock_m);
		EEcopiaARAM(&Clock_Y);

		EEcopiaARAM(&freqRange);

		}
	else {
		LuceLCD=7;
		ContrLCD=10;

		freqRange=1;

		EEcopiaAEEPROM(&ContrLCD);
		EEcopiaAEEPROM(&LuceLCD);
		EEcopiaAEEPROM(&freqRange);
		EEscrivi(&EECheck,0x47);

		}


	aggLuceContrasto();

	print_init();

//	LCDX=0;
//	LCDXY(5);						// X,Y=[2,1]
// fare??	LCDXY2(5,0);						// X,Y=[2,1]





	m_Led1Bit=1;




	FLAGS=0;						// salvare in EEPROM?



warm_reset:



//		goto main;





	INTCONbits.GIE = 1;			// attiva interrupt globali
	INTCONbits.GIEL = 1;			// attiva interrupt low-pty


	Counter=0;
	CounterL2=0;
#ifdef USA_232
	*((byte *)&CommStatus)=0;
#endif


#ifdef USA_232
	RCSTAbits.CREN=1;			// abilito ricevitore 232
#endif


	ClrWdt();							// rientro caldo (da watchdog):
	KBInit();

	LCDInit();				// ripristino display... (ANCHE sopra?)

	aggLuceContrasto();

	print_init();


	// ritardo x splash screen
	Delay_mS(250);
				m_Led1Bit ^= 1;
	Delay_mS(250);
				m_Led1Bit ^= 1;
	Delay_mS(250);
				m_Led1Bit ^= 1;
	Delay_mS(250);
				m_Led1Bit ^= 1;



	ClrWdt();				// DOPO!


	frequency=0;
	frequencyTempHigh=0;
	frequencyToAdd=0;
	freqRange=1;			// default 100mS = /10
	sampleDivider=SampleDividers[freqRange];

	m_Strobe1Bit = 0;
	m_Strobe2Bit = 1;



	FLAGS=0;						// salvare in EEPROM?


	if(!m_Puls2Bit) {		// cold


debug_init:

		LCDPutChar('*');
#ifdef MOSTRA_OROLOGIO
		FLAGS |= (1 << CLOCK_ON);
#endif


		}

//	call Delay_S








#ifdef USA_232
	PIE1bits.RCIE=1;				// ...e IRQ 232
	Buf232Ptr=0;
	Buf232PtrI=0;
	RCSTAbits.CREN=1;				// abilito ricevitore 232...
#endif

#ifdef USA_232
	sendQueryStatus232();
#endif



	Counter=0x7fff;
	CounterL2=8;

	while(1) {

main:

		ClrWdt();	

		CounterL2--;        // quando il sub-contatore e' zero...
		if(!CounterL2) {
			CounterL2=8;

			Counter--;         // quando il contatore e' zero...
			if(!Counter) {
				Counter=0x7fff;
				goto ResCnt;
				}
			}



		goto main;


ResCnt:


//	m_Led0Bit ^= 1;


		if((FLAGS & (1 << CLOCK_ON)) ) {


		// set XY [1:0]
			LCDOutCmd(0x81);            // mando comando

			LCDPrintDec2(Clock_H);
			LCDOutData(':');
			LCDPrintDec2(Clock_M);
			LCDOutData(':');
			LCDPrintDec2(Clock_S);
			LCDXY_();								// rimetto a posto il cursore...

			}

		LCDCls();
		if(freqRange == 1)
			frequency *= 10;
		if(m_ModeBit)			// prescaler
			frequency *= 10;
		ltoa(frequency,buffer);
//			sprintf(buffer,"%-0+9ld",frequencyToAdd);
		i=strlen(buffer);
		LCDX=12-i;
		LCDXY_();
		LCDWrite(buffer);
		LCDWriteROM(" Hz");

//		frequency=0;


		if(kbKeys[2] & 0x10) {
			LCDCls();
			LCDWriteROM(CopyrDate);

			while(kbKeys[2]) 
				ClrWdt();
			}

		if((kbKeys[1] & 0x1) /* && !(kbKeys[1] & 0x2) */ ) {
			freqRange++;
			freqRange &= 1;			// OCCHIO!

			WriteTimer1(0);
			frequency=0;
			sampleDivider=SampleDividers[freqRange];

			LCDCls();
			LCDWriteROM(freqRange==0 ? (const rom char *)"x1" : (const rom char *)"x10");

			for(Counter=0; Counter<0xffff; Counter++)
				ClrWdt();
			}


		if(kbKeys[0] & 0x10) {		// entra in impostazione M.F.

			LCDCls();
//			sprintf(buffer,"%-0+9ld",frequencyToAdd);
			ltoa(frequencyToAdd,buffer);
			if(frequencyToAdd<0)
				LCDPutChar('-');
			else
				LCDPutChar('+');

			LCDWrite(buffer);
			LCDWriteROM(" Hz");
			
			while(kbKeys[0]) 
				ClrWdt();
			}


		if(TimerBuzz) {
			if(!--TimerBuzz) {

				// spengo PWM1 (faccio in modo che non si attivi mai... v. anche PR2)
//			CCPR1L=255;
//	movlw  00000011b
//	movwf  T2CON

				}
			}


		}			// while(1)


#ifdef USA_232
	CloseUSART();
#endif

    
	} //end main







// ---------------------------------------------------------------------------------






// ---------------------------------------------------------------------



void MandaTastoPremuto(byte n) {				// entra W=temp=char

#ifdef USA_232
	PutByte232(n);
#endif

	if(!m_Puls2Bit)
		LCDPutChar(n);

	//goto  MandaTastoPremuto2;

	}

void MandaTastoPremuto2(void) {
	}

void MandaTastoRilasciato(byte n) {
	}




// ---------------------------------------------------------------------

void StdBeep(void) {
	word i;

	for(i=0; i<1000; i++) {
		m_Led0Bit ^=1;
		Delay_mS(1);
		}

	}

void ErrorBeep(void) {
	word i;

	for(i=0; i<1000; i++) {
		m_Led0Bit ^=1;
		Delay_mS(2);
		}
	}



// ---------------------------------------------------------------------

#ifdef USA_232

short int GetByte232(void) {                // torna 0 e NC se niente oppure C e il byte in A
	byte ch;

  ClrWdt();
	if(Buf232Ptr == Buf232PtrI) {
		return -1;
		}

	ch=Buf232[Buf232PtrI++];
	Buf232PtrI &= 0xf;				// max 16
	return ch;
	}

void PutByte232(byte Byte2Send) {                // manda un byte

	do {
		ClrWdt();
		} while(!PIR1bits.TXIF);			// aspetto che il buffer di TX sia libero (v. anche MandaHdr)

	WriteUSART(Byte2Send);

	do {
		ClrWdt();
		} while(BusyUSART());

	}

#endif

// ---------------------------------------------------------------------


#ifdef USA_232
void sendQueryStatus232(void) {

	PutByte232(ESC);			//;VT100
	PutByte232('[');

#ifdef USA_TASTIERA	
	PutByte232(1);
#else
#ifdef USA_ANALOG
	PutByte232(2);
#else
	PutByte232(0);
#endif
#endif

	PutByte232(0);

	PutByte232(LCDCols);					//XSize
	PutByte232(LCDRows);					//YSize
	
	// VT100
	PutByte232('0');
	PutByte232('c');

	}
#endif



// ---------------------------------------------------------------------

#define Delay01() Delay_uS(1		/* TIME_GRANULARITY*/)			// 1 bit-time



//Delays approx 2*W milliseconds
void Delay_mS(byte mSec2) {
	
	while(mSec2--) {

		// usare Delay10KTCY();			// TARARE!! @20MHz??

		Delay_uS(200);
		Delay_uS(200);
		Delay_uS(200);
		Delay_uS(200);
		Delay_uS(200);
		Delay_uS(200);
		Delay_uS(200);
		Delay_uS(200);
		Delay_uS(200);
		Delay_uS(200);
		}

//	return;
	}


//Delays W microseconds (includes movlw, call, and return) @ 24MHz
void Delay_uS(byte uSec) {

	do {
		Delay1TCY();			// TARARE!! @40MHz??
		Delay1TCY();
		Delay1TCY();
		ClrWdt();
		Delay1TCY();
		Delay1TCY();
		} while(--uSec);

  //return             ; 1
	}


  

void Delay_S_(byte n) {				// circa ?s @ 40MHz !! modificare!!
	byte i=16,i2;

	do {

Delay_S_1:								// 256x256uS= ~66mS
		i2=0;

Delay_S_2:
		do {
Delay_S_3:							// 5x200nS=1uS, x 256=256uS
			Delay10TCY();			// TARARE!! @40MHz??
			ClrWdt();			// 1

			} while(i2--);

		} while(i--);

  //return             ; 1

	}










#ifdef USA_232

// ---------------------------------------------------------------------
// calcola la parita: entra W, esce 0 se E e 1 se O
byte GetParita(byte n) {
	byte p=0;

#ifdef DARIO
	p=n >> 1;			// Microchip AN774
	p ^= n;
	n= p >> 2;
	p ^= n;
	n= p >> 4;
	p ^= n;
	return (p & 1);
#endif

	if(n & 1)
		p++;
	if(n & 2)
		p++;
	if(n & 4)
		p++;
	if(n & 8)
		p++;
	if(n & 16)
		p++;
	if(n & 32)
		p++;
	if(n & 64)
		p++;
	if(n & 128)
		p++;
  return p;
	}


#endif




// -------------------------------------------------------------------------------------

                              // ...o solo in EPROM!
void EEscrivi_(SHORTPTR addr,byte n) {	// non uso puntatore, xch va a finire a 16bit! SPRECONI!

	EEADR = (byte)addr;
	EEDATA=n;

	EECON1bits.EEPGD=0;		// Point to Data Memory
	EECON1bits.CFGS=0;		// Access EEPROM
	EECON1bits.WREN=1;

	INTCONbits.GIE = 0;			// disattiva interrupt globali
	EECON2=0x55;		 // Write 55h
	EECON2=0xAA;		 // Write AAh
	EECON1bits.WR=1;									// abilita write.
	INTCONbits.GIE = 1;			// attiva interrupt globali
	do {
		ClrWdt();
		} while(EECON1bits.WR);							// occupato ? 


	EECON1bits.WREN=0;								// disabilita write.
  }

byte EEleggi(SHORTPTR addr) {

	EEADR=(byte)addr;			// Address to read
	EECON1bits.EEPGD=0;		// Point to Data Memory
	EECON1bits.CFGS=0;		// Access EEPROM
	EECON1bits.RD=1;		// EE Read
	return EEDATA;				// W = EEDATA
	}











/** EOF frequenz.c *********************************************************/
