/*********************************************************************
 *
 *                Microchip USB C18 Firmware - Mouse Interlink + PS2
 *
 *********************************************************************
 * FileName:        frequenz.h
 * Dependencies:    See INCLUDES section below
 * Processor:       PIC18
 * Compiler:        C18 2.30.01+
 * Company:         Microchip Technology, Inc.
 *
 *
 * Author               Date        Comment
 *~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 * Rawin Rojvanit       11/19/04    Original.
 ********************************************************************/

#ifndef FREQUENZ_H
#define FREQUENZ_H



//#define USA_232	1							// I/F 232 
#define DEBUG_PULS  1					// reset ai default di alcuni valori se premuto il puls2 all'accensione
//#define USA_KB_BUZZER 1					// bip di conferma sui tasti/touch_screen
//#define I2C_MSSP_HARDWARE	


#if defined(__18CXX)
#define TMR1BASE_100  (65536-(5000/1)+9)		// IRQ ogni 4mS circa (prescaler=8: 100nS*8=800, 40000000/800=5000 0 ???)
#define TMR1BASE_1000  (65536-(50000/1)+9)		// IRQ ogni 40mS circa (idem)
#else
#define TMR1BASE_100  (65536-(510/1)+0)		// IRQ ogni 1mS circa (prescaler=1: 125nS*1=125, 4000000/1000=32000)
#define TMR1BASE_1000  (65536-(40000/1)+29)		// IRQ ogni 10mS circa (idem)
#endif

#define TIME_GRANULARITY 104   // il tempo per 1 bit (104.2 @ 9600baud)

#define MAX_TASTI_COMTEMPORANEI 3



#define SERNUM 1000
#define VERNUMH 1
#define VERNUML 3



#define LCD_MAX_ROWS 1
#define LCD_MAX_COLS 16				// reali (il display  settato a LCD_TEXT_COLS!)




struct COMM_STATUS
{
// errori/flag in CommStatus Seriale
	unsigned int FRAME_2SEND:1;         // c'e' da mandare un frame
	unsigned int WAIT_4ACK  :1;
	unsigned int FRAME_REC  :1;         // c'e' un frame ricevuto
	unsigned int COMM_OVRERR:1;			// overrun seriale 
	unsigned int COMM_OVL   :1;         // errore di overflow buffer in ricezione 
	unsigned int COMM_TOUT  :1;         // errore di timeout in ricezione frame
	unsigned int COMM_FRERR :1;         // errore di framing (START/STOP) in ricezione 
	unsigned int COMM_PERR  :1;         // errore di parita' in ricezione 
};


// Flag bits (FLAGS)
#define NOSCANK	0
#define NOKCLICK	1
#define CLOCK_ON		3
#define IN_ESCAPE_CHAR  4
#define BUZZON		5
#define TRIGK		7


enum {
	BEEP_STD_FREQ	=80,				// 150=2KHz ca. @ 20MHz i buzzer danno pi dB verso i 4KHz...
	BEEP_LOW_FREQ	=200,
	BEEP_HIGH_FREQ	=50
	};


/** P U B L I C  P R O T O T Y P E S *****************************************/


void DoI2CStart(void);
#define DoI2CMO(n) I2CTXByte(n)
void DoI2CStop(void);
byte I2CBITIN(void);
byte I2CRXByte(void);
byte I2CRX1Byte(void);
byte I2CTXByte(byte);
byte I2CRead16Seq(word,byte);
byte I2CRead8Seq(byte,byte);
#define I2CRead16SeqDefault() I2CRead16Seq(0 ,16 /*FINIRE*/ /*I2CBuffer*/)
byte I2CWritePagePoll(void);
byte I2CWritePage16(word,byte);
void I2CWritePage(byte,byte);

void Delay_mS(byte );
void Delay_uS(byte );
void Delay_S_(byte);
#define Delay_S() Delay_S_(1)				// circa 1s @ 20MHz !! 

#define EEscrivi0(p) EEscrivi(p,0)               // scrive 0...

#define EEscrivi(a,b) { *((byte *)a)=b; EEscrivi_(a,b); }  // scrive W in RAM e EPROM...
																// entra FSR alla loc. memoria (RAM/EPROM) voluta
void EEscrivi_(SHORTPTR,byte);
byte EEleggi(SHORTPTR);
#define EEcopiaARAM(p) { *p=EEleggi(p); }
#define EEcopiaAEEPROM(p) EEscrivi_(p,*p)


void print_init(void);

void aggLuceContrasto(void);


void KBInit(void);
byte checkKey(void);

#define kbKeyPressed()  ( kbKeys[0] )
	// ritorna NOZERO se c'e' un tasto premuto
byte GetKBchar(byte);

void SetBeep(void);

#endif // FREQUENZ_H
