/*********************************************************************
 *
 *                Microchip XC8 Firmware - PICFrequency
 *
 *********************************************************************
;    Filename:	    Frequenz.c	(Frequenzimetro come da Kit LX1525 di N.E. rivista 213 gennaio 2003) *
;    Date:          25/4/17                                          *
;    File Version:  1.03														              *
;                                                                     *
;    Author:        Dario/Cinzia Greggio                              *
;    Company:       ADPM Synthesis sas                                *
 * Dependencies:    See INCLUDES section below
 * Processor:       PIC16
 * Compiler:        XC8 1.30+
 * Company:         Microchip Technology, Inc.
 *
 * Author               Date        Comment
 *~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 * Dario Greggio        24/04/06    .
 * Cinzia Greggio       28/04/17    rewrite per 16F1847 e XC8 1.41
 ********************************************************************/

/** I N C L U D E S **********************************************************/
#include <xc.h>
#include "plib.h"
#include <string.h>
#include <stdlib.h>
#include <stdio.h>
#include "typedefs.h"


#include "frequenz.h"
#include "io_cfg.h"             // I/O pin mapping
#include "lcd.h"


#pragma config WDTE = ON, MCLRE=OFF, LVP=OFF
//#pragma config FOSC = HS, FCMEN=ON, CLKOUTEN=OFF, IESO=ON, PWRTE=ON, BOREN=ON
#pragma config FOSC = INTOSC, PLLEN=ON, FCMEN=ON, CLKOUTEN=OFF, IESO=ON, PWRTE=ON, BOREN=ON		// PLL anche sotto


#define _XTAL_FREQ 32000000L
#define FCY 32000000L


/** V A R I A B L E S ********************************************************/

byte InitOK;			// idem...

extern byte LCDCols,LCDRows;	
extern byte LCDX,LCDY;	
extern byte LuceLCD,ContrLCD,StatoLCD;


extern byte kbKeys[MAX_TASTI_COMTEMPORANEI];
extern byte kbKeysOld[MAX_TASTI_COMTEMPORANEI];
extern byte kbScanCode;



byte EECheck;


volatile unsigned long int frequency;
long int frequencyToAdd;
char frequencyToAddBuf[10];
dword frequencyTempHigh;
word sampleDivider;
byte freqRange,mode,inFreqSet,freqSetPos;


static const char FREQUENZ_C[]="$Id:FREQUENZ16.C Ver 1.0.3 - 29/11/17";
static const word SampleDividers[]={ 100,10 };
extern const char CopyrDate[];

// ---------------------------------------------------------------------




extern byte FLAGS;        //GENERAL PURPOSE FLAG


/** P R I V A T E  P R O T O T Y P E S ***************************************/
void doMain(void);

void MandaTastoPremuto(byte);
void MandaTastoPremuto2(void);
void MandaMousePremuto(void);
void MandaTastoRilasciato(byte);

short int GetByte232(void);
void PutByte232(byte);
void sendQueryStatus232(void);


/******************************************************************************
 * Function:        void (void)
 *
 * PreCondition:    None
 *
 * Input:           None
 *
 * Output:          None
 *
 * Side Effects:    None
 *
 * Overview:        
 *                  
 *
 * Note:            None
 *****************************************************************************/


// METTERE un FLAG per disattivare scroll automatico (CHR$13) su ultima riga, per sfruttarla meglio??


//	__IDLOCS 0x4744

// int osc 32MHZ 
// 125nS/istruzione @ 32MHz





#ifdef USA_232

struct COMM_STATUS CommStatus;
byte Byte2Send;
byte ByteRec;
byte Buf232Ptr;					//contatore per scrittura buffer 232
byte Buf232PtrI;				//contatore per lettura buffer 232
byte Buf232[16];								// buffer 16 byte per ricezione I2C e 232 (BANCO 1)

#endif

char buffer[20];



//#ifdef __16F74				; I2C slave software, solo 74 ... NO! usati come temp in RX anche da 874!!
extern byte I2CData;
extern byte I2CCnt;
//#endif




#define CR          0xd
#define LF          0xa
#define BEL					0x7
#define BS					0x8
#define TAB					0x9
#define DC1					0x11
#define DC2					(DC1+1)
#define DC3					(DC1+2)
#define DC4					(DC1+3)
#define FF					0xc
#define CAN					0x18				// stesso di FF ossia CLS
#define ESC					0x1b
// VTAB, curs giu', curs dx??

#define FLASH_TIME  7



//
//***** VARIABLE DEFINITIONS

//
word Counter;
byte CounterL2;
volatile byte Timer1;
extern byte FLAGS_ESC;					// per sequenze ESCAPE: 0=SetLuce, 1=SetContr, 2=SetFreq; 3=SetLed; 4&5=SetXY (1&2); 6=SetKClick
//word Tmr1Base;


extern byte TimerBuzz;
extern byte BeepFreq;



byte Clock_S;			// orologio hh:mm:ss gg:mm:yyyy, mirrored in EEPROM
byte Clock_M;
byte Clock_H;
byte Clock_D;
byte Clock_m;
byte Clock_Y;
byte Clock_10;		// (decimi di secondo per dividere interrupt); ANCHE MARKER_GOOD_EEPROM (solo in Eprom, v.)



// Flag bits (FLAGS)
#define NOSCANK	0
#define NOKCLICK	1
#define CLOCK_ON		3
#define IN_ESCAPE_CHAR  4
#define BUZZON		5
#define TRIGM2	6
#define TRIGK		7
#define TRIGM		7



//**********************************************************************

//extern void _startup (void);        // See c018i.c in your C18 compiler dir
//#pragma code _RESET_INTERRUPT_VECTOR = 0x000800
//void _reset(void) {
//  _asm goto _startup _endasm
//	}

	/****************************************************************************
*
*  high_interrupt
*  Descrizione: gestore dell'interrupt
*  Riferimento:
*  Parametri in ingresso: nessuno
*  Parametri in uscita: nessuno
*  Note:
*
****************************************************************************/

//#pragma interrupt high_isr nosave=section("MATH_DATA"),section(".tmpdata")
void interrupt high_isr(void) {

	if(INTCONbits.TMR0IF) {
		INTCONbits.TMR0IF=0;
		frequencyTempHigh++;
		}

	if(PIR3bits.TMR4IF) {
		PIR3bits.TMR4IF = 0;
		m_Strobe1Bit ^= 1;
		}	

	if(PIR1bits.TMR2IF) {
		PIR1bits.TMR2IF = 0;

//		TMR2=0;			// re-inizializzo TMR non serve qua!

//		m_Strobe1Bit ^= 1;

		if(!--sampleDivider) {
			frequency = TMR0;
			sampleDivider=SampleDividers[freqRange];
			if(mode)
				sampleDivider*=10;		// passo da 100mS/1s a 1s/10s per maggior precisione
//			frequency = frequencyTempHigh;
//			frequency <<= 16;
//			*((word *)(((char *)&frequency)+1)) = TMR1;
			*((dword *)(((char *)&frequency)+1)) = frequencyTempHigh;
			TMR0=0;
//			TMR1=0;
			frequencyTempHigh=0;
			m_Led0Bit ^= 1;			//10 (o 100mS) 28.11.17
			}


		Timer1++;				// 10mSec

#ifdef USA_KB_BUZZER
		if(!(Timer1 & 0x3)) {		//16mSec ca.
			if(FLAGS & (1 << BUZZON)) {				// OCCHIO alla contemporaneita' col BEEP (chr$7)
		//		CCPR1L=255;				// spengo PWM1 (faccio in modo che non si attivi mai... v. anche PR2)
		//	movlw  00000000b
		//	movwf  T2CON
				FLAGS &= ~(1 << BUZZON);
				}
			}
#endif


		if(!(Timer1 & 0x1f) && !(FLAGS & (1 << NOSCANK))) {		//125mSec ca.
		

	//				m_Led2Bit ^= 1;

			ClrWdt();
		
			if(!m_Puls0Bit)	{
				kbKeys[0]++;
				kbKeys[0] |= 1;			// questo indica la pressione (b0)
				kbKeys[0] &= 0x1f;	// questo indica la press. prolungata (b4)
//				checkKey(/*kbScanCode*/);
				}
			else
				kbKeys[0]=0;
			if(!m_Puls1Bit)	{
				kbKeys[1]++;
				kbKeys[1] |= 1;
				kbKeys[1] &= 0x1f;
				}
			else
				kbKeys[1]=0;
			if(!m_Puls2Bit)	{
				kbKeys[2]++;
				kbKeys[2] |= 1;
				kbKeys[2] &= 0x1f;
				}
			else
				kbKeys[2]=0;

		
			FLAGS |= (1 << TRIGK);
		
			}			// NOSCANK


			
		if(!(Timer1 & 0x3f)) {		//256mSec ca.

	// avendo CLRWDT nell'IRQ, in pratica non salta MAI!!

			Clock_10++;
			if(Clock_10 == 4) {			// FINIRE!!
				Clock_10=0;
				Clock_S++;
				if(Clock_S == 60) {
					Clock_S=0;
					Clock_M++;
					if(Clock_M == 60) {
						Clock_M=0;
						Clock_H++;
						if(Clock_M == 24) {
							Clock_H=0;
							}
						}
					}
				}

			}

		}		// PIR1IF...

	if(/*PIR1bits.TXIF || */
		PIE1bits.RCIE && PIR1bits.RCIF)	{	// serve il controllo su RCIE, se abiliti e disabiliti IRQ da programma...

#ifdef USA_232
	//	bcf  PIR1,RCIF

		if(RCSTAbits.FERR)
			CommStatus.COMM_FRERR=1;
		if(RCSTAbits.OERR) {
			RCSTAbits.CREN=0;
			Nop(); Nop();
			RCSTAbits.CREN=1;								// forse basta BCF (v. datasheet), anche se non e' chiaro...
	//	bsf CommStatus,COMM_OVRERR				; serve l'overrun??
			}


	//	bsf CommStatus,COMM_OVL						; gestire overflow buffer?
		ByteRec=ReadUSART();					// QUESTO pulisce interrupt!
																	// salvo cmq anche qui...
		Buf232[Buf232Ptr++]=ByteRec;				// max 16
		Buf232Ptr &= 0xf;
#endif
	
		}

	}




/** D E C L A R A T I O N S **************************************************/
void main(void) {
	short int ch;
	byte i,j;
	byte *p;
	long myval;



/*========================================================================
;       RESET ENTRY
;========================================================================*/
start:
cold_reset:

	InitOK=PCON 	/*STATUS*/;
//	StatusReset();

//	INTCONbits.PEIE = 1;		// attiva interrupt perif (SERVE x USART!) NON + v. low/high
//	INTCONbits.GIE = 1;			// attiva interrupt globali


	ClrWdt();					// DOPO!

	PCONbits.nBOR=1;
	PCONbits.nPOR=1;


	OSCCONbits.IRCF=0b1110;
	OSCCONbits.SPLLEN=1;		// v. anche CONFIG
	while(!OSCSTATbits.PLLR)
		ClrWdt();


	OpenTimer0(TIMER_INT_ON & T0_16BIT & T0_SOURCE_EXT & T0_PS_1_1 /*& T0_EDGE_FALL*/);		// SEMPRE high pty!
//	OpenTimer1(TIMER_INT_ON & T1_SOURCE_INT & T1_OSC1EN_OFF & T1_PS_1_1 & T1_SYNC_EXT_OFF);
	T1CON= 0b00000001;
	T1GCON=0		/*0b10010001*/;
	T2CON=0b00111111;		// 1:8 Pre, 1:64 Post
	PR2=155;			// 50Hz
  PIE1bits.TMR2IE=1;

#ifdef TEST
	T4CON=0b00111101;			// 1:8 Pre, 1:4 Post
	PR4=99;			//1250Hz per test (400uS)
  PIE3bits.TMR4IE=1;
#endif



	EnablePullups();

// SE si usa warm/cold reset, lo stato degli I/O va ripristinato dopo ogni reset!!!

	ANSELA=0b00000000;	//
	ANSELB=0b00000000;	//
//	CMCON=0x7;

	WPUA=0b00000000;
	WPUB=0b00000111;
	OPTION_REGbits.nWPUEN=0;


	LATA=0b00000000;				// 
	LATB=0b00000000;


	TRISA=0b00110000;					// TimerInput, 1 tasto, 2 strobe, 1 led
	TRISB=0b00000111;					// output cmd LCD, 2 tasti



	WriteTimer0(0);					//clr tmr0 & prescaler
//	WriteTimer1(0);					//clr tmr1

//	movlw	00000111b	; pull-ups B, tmr0 enable, prescaler Timer with 1:256 prescaler FINIRE!!
//	movwf	OPTION_REG

	INTCONbits.GIE=0;			// disabilito tutti interrupt
	INTCONbits.PEIE = 0;			// 


	ClrWdt();

#ifdef USA_232

	OpenUSART(USART_RX_INT_ON & USART_TX_INT_OFF & USART_ASYNCH_MODE & 
		USART_EIGHT_BIT & USART_CONT_RX & USART_BRGH_HIGH,
		23);
//	INTCONbits.PEIE = 1;			// attiva interrupt perif (SERVE x USART!
	INTCONbits.PEIE = 1;			// 

// v. skyusb18.c, e/oppure usare irq low/high
//	baudUSART(BAUD_16_BIT_RATE & BAUD_WAKEUP_OFF & BAUD_AUTO_OFF);
	// x= (Fosc / (Brate * 16 (BRGH=1) ) ) ) -1
	// Fosc=44236800 (11.0592 XTAL)
	
#endif

//	Tmr1Base=TMR1BASE_100;
//	WriteTimer1(Tmr1Base);					// inizializzo TMR1

	BeepFreq=BEEP_STD_FREQ;

// il timer 2  usato per il beep e il PWM...
//	CCP1CON=0b00001100;			//			 TMR2: PWM mode
	CCP1CON=0b00000000;			//			 NO: RB3 mi SERVE!
//	OpenPWM();
//	CCPR1L=255;
//	movlw		255
//	movwf	 CCPR1H				; questo  read-only in modalit PWM

//	T2CON=0x7;	//00000111b		// accendo il timer2, il che attiva entrambi i PWM; prescaler=16
//	T2CON=0b00001111;		// accendo il timer2, il che attiva entrambi i PWM; prescaler=16; postscaler=1:2


//	PR2=BEEP_STD_FREQ;

	

	ClrWdt();

init_i2c:
#ifdef I2C_MSSP_HARDWARE			// default su 874...
															// per ora uso solo SW, su 74... FINIRE (se funziona!)!

	SSPADD=(((40000000/100000)/4) -1);    // read selected baud rate (100KHz)
	//; initialize I2C baud rate
	SSPSTAT=0b00000000;	            // select I2C input levels; enable slew rate

#ifdef I2C_MSSP_HARDWARE	
	SSPCON2=0;
#endif

#ifdef __16F74								//
	SSPCON=0b00111011;							// Firmware Controlled Master Mode, Slave Idle
#else
	SSPCON=0b00111000;							// Master mode, SSP enable
// AN578 mette 00111110b...
//	SSPCON=0b00111110;							// Master mode, SSP enable
#endif

#endif

//	OpenI2C()



	if(InitOK & (1 << 3 /*NOT_TO*/))	 	// se era watchdog...
		{

		}


	LCDInit();				// ripristino display... (ANCHE dopo WARM_RESET?)


	if(EEleggi(&EECheck) == 0x47) {				// se  gi stata inizializzata, salto!
		EEcopiaARAM(&LuceLCD);
		EEcopiaARAM(&ContrLCD);

		EEcopiaARAM(&Clock_S);
		EEcopiaARAM(&Clock_M);
		EEcopiaARAM(&Clock_H);
		EEcopiaARAM(&Clock_D);
		EEcopiaARAM(&Clock_m);
		EEcopiaARAM(&Clock_Y);

		EEcopiaARAM(&freqRange);

		}
	else {
		LuceLCD=7;
		ContrLCD=10;

		freqRange=1;

		EEcopiaAEEPROM(&ContrLCD);
		EEcopiaAEEPROM(&LuceLCD);
		EEcopiaAEEPROM(&freqRange);
		EEscrivi(&EECheck,0x47);

		}


	aggLuceContrasto();

	print_init();

//	LCDX=0;
//	LCDXY(5);						// X,Y=[2,1]
// fare??	LCDXY2(5,0);						// X,Y=[2,1]



	m_Led1Bit=1;


	FLAGS=0;						// salvare in EEPROM?


warm_reset:

//		goto main;

	INTCONbits.GIE = 1;			// attiva interrupt globali
	INTCONbits.PEIE = 1;			// 


	Counter=0;
	CounterL2=0;
#ifdef USA_232
	*((byte *)&CommStatus)=0;
#endif


#ifdef USA_232
	RCSTAbits.CREN=1;			// abilito ricevitore 232
#endif


	ClrWdt();							// rientro caldo (da watchdog):
	KBInit();

	LCDInit();				// ripristino display... (ANCHE sopra?)

	aggLuceContrasto();

	print_init();


	// ritardo x splash screen
	Delay_mS(250);
				m_Led1Bit ^= 1;
	Delay_mS(250);
				m_Led1Bit ^= 1;
	Delay_mS(250);
				m_Led1Bit ^= 1;



	ClrWdt();				// DOPO!


	inFreqSet=0; freqSetPos=0;
	frequency=0;
	frequencyTempHigh=0;
	frequencyToAdd=0;
	freqRange=1;			// default 100mS = /10
	mode=0;		// frequenzimetro
	sampleDivider=SampleDividers[freqRange];

	m_Strobe1Bit = 0;
	m_Strobe2Bit = 1;



#ifdef TEST
	m_Strobe1Bit = 1;
	m_Strobe2Bit = 0;
#endif


	FLAGS=0;						// salvare in EEPROM?


	if(!m_Puls2Bit) {		// cold


debug_init:

		LCDPutChar('*');
#ifdef MOSTRA_OROLOGIO
		FLAGS |= (1 << CLOCK_ON);
#endif

		}



#ifdef USA_232
	PIE1bits.RCIE=1;				// ...e IRQ 232
	Buf232Ptr=0;
	Buf232PtrI=0;
	RCSTAbits.CREN=1;				// abilito ricevitore 232...
#endif

#ifdef USA_232
	sendQueryStatus232();
#endif



	Counter=0x7fff;
	CounterL2=8;

	while(1) {

main:

		ClrWdt();	

		CounterL2--;        // quando il sub-contatore e' zero...
		if(!CounterL2) {
			CounterL2=8;

			Counter--;         // quando il contatore e' zero...
			if(!Counter) {
				Counter=0x7fff;
				goto ResCnt;
				}
			}

		continue;


ResCnt:


//	m_Led0Bit ^= 1;


		if((FLAGS & (1 << CLOCK_ON)) ) {

		// set XY [1:0]
			LCDOutCmd(0x81);            // mando comando

			LCDPrintDec2(Clock_H);
			LCDOutData(':');
			LCDPrintDec2(Clock_M);
			LCDOutData(':');
			LCDPrintDec2(Clock_S);
			LCDXY_();								// rimetto a posto il cursore...

			}

		if(!inFreqSet) {
			LCDCls();
			myval=frequency;
			if(mode) {			// periodo
				if(freqRange == 0)
					myval /= 10;
				if(m_ModeBit)			// prescaler
					myval *= 10;
				if(myval)
					myval=1000000UL/myval;
				else
					myval=0;
				}
			else {
				if(freqRange == 1)
					myval *= 10;
				if(m_ModeBit)			// prescaler
					myval *= 10;
				if(frequencyToAdd) {
					LCDPutChar('*');
					myval += frequencyToAdd;
					}
				}
			if(myval>=0) {
				if((*(((char *)&frequency)+3)) < 3) {			// circa 50.000.000 !
//				if(HIBYTE(HIWORD(frequency)) < 3) {			// circa 50.000.000 !
					ltoa(buffer, myval, 10);
		//			sprintf(buffer,"%-0+9ld",frequencyToAdd);
					i=strlen(buffer);
	//				i+=(i-1)/3;			// creo lo spazio per i punti decimali
					LCDX=12-i-((i-1)/3);
					LCDXY_();
			//		LCDWrite(buffer);
					for(j=0; i; j++) {
						LCDPutChar(buffer[j]);
						i--;
						if(i && !(i % 3))
							LCDPutChar('.');
						}
					}
				else {
					LCDX=3;
					LCDXY_();
					LCDWrite("Overflow");
					}
				if(mode)
					LCDWriteROM(" uS");
				else
					LCDWriteROM(" Hz");
				}
			else {
				LCDX=4;
				LCDXY_();
				LCDWrite("Fin < MF");
				}
				
	
			}


		if(kbKeys[2] & 0x1) {
			if(inFreqSet) {
				freqSetPos++;
				if(freqSetPos>8)
					freqSetPos=0;
				goto updFreqSet;
				}	
			else {
				mode ^= 1;
				LCDCls();
				LCDWrite(mode==0 ? (const char *)"Frequenza" : (const char *)"Periodo");

				goto updDaCambio;
	
				}
			}
		if(kbKeys[2] & 0x10) {
			if(!inFreqSet) {
				LCDCls();
				LCDWriteROM(CopyrDate);
	
				while(kbKeys[2]) 
					ClrWdt();
				}
			}

		if((kbKeys[1] & 0x1) /* && !(kbKeys[1] & 0x2) */ ) {
			if(inFreqSet) {
				if(kbKeys[2] & 0x1) {			// azzera se premuti insieme! non va... boh 29.11.17
					frequencyToAdd=0;
					goto updFreqSet2;
					}
				else {
					if(!freqSetPos) {
						frequencyToAddBuf[0] = frequencyToAddBuf[0]=='+' ? '-' : '+';
						}
					else {
						frequencyToAddBuf[freqSetPos]++;
						if(frequencyToAddBuf[freqSetPos] > '9')
							frequencyToAddBuf[freqSetPos]='0';
						}
					goto updFreqSet;
					}
				}	
			else {
				freqRange++;
				freqRange &= 1;			// OCCHIO!

updDaCambio:	
				TMR2=0;			TMR0=0;
				frequency=0;  frequencyTempHigh=0;
				sampleDivider=SampleDividers[freqRange];
//				sampleDivider=1;		// riparte veloce!
				myval=0;
	
				LCDCls();
				LCDWrite(freqRange==0 ? (const char *)"x1" : (const char *)"x10");
	
				__delay_ms(350);
				ClrWdt();
				}
			}


		if(kbKeys[0] & 0x1) {
			if(inFreqSet) {
				inFreqSet=0;
				frequencyToAdd=atol(frequencyToAddBuf);
				LCDX=10;
				LCDXY_();
				LCDWrite(" saved");
				__delay_ms(250);
				LCDOutCmd(0b00001100);
				}	
			}
		if(kbKeys[0] & 0x10 && !mode) {		// entra in impostazione M.F.
			inFreqSet=1;
			freqSetPos=5;		// inizio col cursore su 1k
updFreqSet2:
			ltoa(buffer, abs(frequencyToAdd), 10);
			i=strlen(buffer);
			if(frequencyToAdd>=0) {
				frequencyToAddBuf[0]='+';
				}
			else {
				frequencyToAddBuf[0]='-';
//				i--;
				}
			for(j=1; j<9; j++) {
				frequencyToAddBuf[j]='0';
				}
			for(j=0; j<i; j++) {
				frequencyToAddBuf[8-j]=buffer[i-j-1];
				}
			frequencyToAddBuf[9]=0;
			

updFreqSet:
			LCDCls();
//			sprintf(buffer,"%-0+9ld",frequencyToAdd);
			for(i=0; i<9; i++) 
				LCDPutChar(frequencyToAddBuf[i]);

			LCDWrite(" Hz");
			LCDOutCmd(0b00001101);			// cursore/blink
			LCDX=freqSetPos;
			LCDXY_();
			
			while(kbKeys[0]) 
				ClrWdt();
			}


		if(TimerBuzz) {
			if(!--TimerBuzz) {

				// spengo PWM1 (faccio in modo che non si attivi mai... v. anche PR2)
//			CCPR1L=255;
//	movlw  00000011b
//	movwf  T2CON

				}
			}


		}			// while(1)


#ifdef USA_232
	CloseUSART();
#endif

    
	} //end main







// ---------------------------------------------------------------------------------






// ---------------------------------------------------------------------



void MandaTastoPremuto(byte n) {				// entra W=temp=char

#ifdef USA_232
	PutByte232(n);
#endif

	if(!m_Puls2Bit)
		LCDPutChar(n);

	//goto  MandaTastoPremuto2;

	}

void MandaTastoPremuto2(void) {
	}

void MandaTastoRilasciato(byte n) {
	}




// ---------------------------------------------------------------------

void StdBeep(void) {
	word i;

	for(i=0; i<1000; i++) {
		m_Led0Bit ^=1;
		Delay_mS(1);
		}

	}

void ErrorBeep(void) {
	word i;

	for(i=0; i<1000; i++) {
		m_Led0Bit ^=1;
		Delay_mS(2);
		}
	}



// ---------------------------------------------------------------------

#ifdef USA_232

short int GetByte232(void) {                // torna 0 e NC se niente oppure C e il byte in A
	byte ch;

  ClrWdt();
	if(Buf232Ptr == Buf232PtrI) {
		return -1;
		}

	ch=Buf232[Buf232PtrI++];
	Buf232PtrI &= 0xf;				// max 16
	return ch;
	}

void PutByte232(byte Byte2Send) {                // manda un byte

	do {
		ClrWdt();
		} while(!PIR1bits.TXIF);			// aspetto che il buffer di TX sia libero (v. anche MandaHdr)

	WriteUSART(Byte2Send);

	do {
		ClrWdt();
		} while(BusyUSART());

	}

#endif

// ---------------------------------------------------------------------


#ifdef USA_232
void sendQueryStatus232(void) {

	PutByte232(ESC);			//;VT100
	PutByte232('[');

#ifdef USA_TASTIERA	
	PutByte232(1);
#else
#ifdef USA_ANALOG
	PutByte232(2);
#else
	PutByte232(0);
#endif
#endif

	PutByte232(0);

	PutByte232(LCDCols);					//XSize
	PutByte232(LCDRows);					//YSize
	
	// VT100
	PutByte232('0');
	PutByte232('c');

	}
#endif



// ---------------------------------------------------------------------

#define Delay01() Delay_uS(1		/* TIME_GRANULARITY*/)			// 1 bit-time



//Delays approx 2*W milliseconds
void Delay_mS(byte mSec2) {
	
	while(mSec2--) {

		// usare Delay10KTCY();			// TARARE!! @20MHz??

		Delay_uS(200);
		Delay_uS(200);
		Delay_uS(200);
		Delay_uS(200);
		Delay_uS(200);
		Delay_uS(200);
		Delay_uS(200);
		Delay_uS(200);
		Delay_uS(200);
		Delay_uS(200);
		}

//	return;
	}


//Delays W microseconds (includes movlw, call, and return) @ 24MHz
void Delay_uS(byte uSec) {

	do {
		__delay_us(1);			// TARARE!! @40MHz??
		ClrWdt();
		} while(--uSec);

  //return             ; 1
	}


  

void Delay_S_(byte n) {				// circa ?s @ 40MHz !! modificare!!
	byte i=16,i2;

	do {

Delay_S_1:								// 256x256uS= ~66mS
		i2=0;

Delay_S_2:
		do {
Delay_S_3:							// 5x200nS=1uS, x 256=256uS
			__delay_ms(1);			// TARARE!! @40MHz??
			ClrWdt();			// 1

			} while(i2--);

		} while(i--);

  //return             ; 1

	}










#ifdef USA_232

// ---------------------------------------------------------------------
// calcola la parita: entra W, esce 0 se E e 1 se O
byte GetParita(byte n) {
	byte p=0;

#ifdef DARIO
	p=n >> 1;			// Microchip AN774
	p ^= n;
	n= p >> 2;
	p ^= n;
	n= p >> 4;
	p ^= n;
	return (p & 1);
#endif

	if(n & 1)
		p++;
	if(n & 2)
		p++;
	if(n & 4)
		p++;
	if(n & 8)
		p++;
	if(n & 16)
		p++;
	if(n & 32)
		p++;
	if(n & 64)
		p++;
	if(n & 128)
		p++;
  return p;
	}


#endif




// -------------------------------------------------------------------------------------

                              // ...o solo in EPROM!
void EEscrivi_(SHORTPTR addr,byte n) {	// non uso puntatore, xch va a finire a 16bit! SPRECONI!

	EEADR = (byte)addr;
	EEDATA=n;

	EECON1bits.EEPGD=0;		// Point to Data Memory
	EECON1bits.CFGS=0;		// Access EEPROM
	EECON1bits.WREN=1;

	INTCONbits.GIE = 0;			// disattiva interrupt globali
	EECON2=0x55;		 // Write 55h
	EECON2=0xAA;		 // Write AAh
	EECON1bits.WR=1;									// abilita write.
	INTCONbits.GIE = 1;			// attiva interrupt globali
	do {
		ClrWdt();
		} while(EECON1bits.WR);							// occupato ? 


	EECON1bits.WREN=0;								// disabilita write.
  }

byte EEleggi(SHORTPTR addr) {

	EEADR=(byte)addr;			// Address to read
	EECON1bits.EEPGD=0;		// Point to Data Memory
	EECON1bits.CFGS=0;		// Access EEPROM
	EECON1bits.RD=1;		// EE Read
	return EEDATA;				// W = EEDATA
	}



/** EOF frequenz.c *********************************************************/
