/*********************************************************************
 *
 *                Microchip USB C18 Firmware Version 1.0
 *
 *********************************************************************
 * FileName:        io_cfg.h
 * Dependencies:    See INCLUDES section below
 * Processor:       PIC18
 * Compiler:        C18 2.30.01+
 * Company:         Microchip Technology, Inc.
 *
 * Author               Date        Comment
 *~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 * Rawin Rojvanit       21/07/04    Original.
 * Dario Greggio        28/02/05    .
 ********************************************************************/

#ifndef IO_CFG_H
#define IO_CFG_H

/** I N C L U D E S *************************************************/

/** T R I S *********************************************************/
#define INPUT_PIN           1
#define OUTPUT_PIN          0



/** I N P U T / O U T P U T *****************************************************/
//#define Led1Bit     5		// per causa di ADCON1 (se USATO, v. conf.), questo e' solo IN-analogico (non digitale) opp. OUTPUT!
// port A  ()			// RA4  OpenCollector
	// bit 0..3 sono righe tastiera (out)
#if defined(__18CXX)
#define Strobe1Bit		0			// pulse out che rientra sul timer ... ?
#define m_Strobe1Bit	LATAbits.LATA0
#define Strobe2Bit		1			// enable freq-in al timer (opp. enable pulse out)
#define m_Strobe2Bit	LATAbits.LATA1

#define Puls2Bit		2
#define m_Puls2Bit	PORTAbits.RA2
#define Puls1Bit		3
#define m_Puls1Bit	PORTAbits.RA3

#define ModeBit			5
#define m_ModeBit		PORTAbits.RA5

#else

#define Strobe1Bit		2			// pulse out che rientra sul timer ... ?
#define m_Strobe1Bit	LATAbits.LATA2
#define Strobe2Bit		3			// enable freq-in al timer (opp. enable pulse out)
#define m_Strobe2Bit	LATAbits.LATA3

#define Puls2Bit		0
#define m_Puls2Bit	PORTBbits.RB0
#define Puls1Bit		1
#define m_Puls1Bit	PORTBbits.RB1

#define ModeBit			5
#define m_ModeBit		PORTAbits.RA5
#endif

#define m_Led1Bit		m_Led0Bit




// port B
// 4 bit alti => I/O LCD
#if defined(__18CXX)
#define Puls0Bit		0
#define m_Puls0Bit	PORTBbits.RB0
#define Led0Bit		1
#define m_Led0Bit		LATBbits.LATB1
#else
#define Puls0Bit		2
#define m_Puls0Bit	PORTBbits.RB2
#define Led0Bit		3
#define m_Led0Bit		LATBbits.LATB3
#endif

#if defined(__18CXX)
#define LCDRSBit	3
#define LCDRWBit	
#define LCDEnBit	2
#define m_LCDRSBit	LATBbits.LATB3
#define m_LCDRWBit	LATBbits.LATB?
#define m_LCDEnBit	LATBbits.LATB2
#else
#define LCDRSBit	1
#define LCDRWBit	
#define LCDEnBit	0
#define m_LCDRSBit	LATAbits.LATA1
#define m_LCDRWBit	LATBbits.LATB?
#define m_LCDEnBit	LATAbits.LATA0
#endif

#define LCDRSVal	(1 << LCDRSBit)
#define LCDRWVal	(1 << LCDRWBit)
#define LCDEnVal	(1 << LCDEnBit)


// port C
#define LCDLuceBit	1		// PWM2 x luce
#define m_LCDLuceBit	LATCbits.LATC1
#define BeepBit			2
#define m_BeepBit		LATCCbits.LATC2

#define I2CDataTris			TRISBbits.TRISB4		// per SLAVE!!





//RXVal       equ 1 << RXBit
//TXVal       equ 1 << TXBit
#define Led1Val     (1 << Led1Bit)
#define I2CslvCVal	(1 << I2CslvCBit)
#define I2CslvDVal	(1 << I2CslvDBit)
#define ChiaveVal		(1 << ChiaveBit)
#define Puls1Val		(1 << Puls1Bit)








// port C
#define TXBit			6		// TX-232
#define RXBit			7		// RX-232
#define TXVal			(1 << TXBit)
#define RXVal			(1 << RXBit)

#define I2CslvCBit	3		// Clock I2C slave
#define m_I2CslvCBit	PORTBbits.RB3
#define I2CslvDBit	4		// Dati I2C slave
#define m_I2CslvDBit	PORTBbits.RB4
// PWM2 x luce
#define LCDLuceBit	1		// PWM2

#define BuzzBit		2		// PWM1

#define Puls1Val  (1 << Puls1Bit)
#define Puls2Val	(1 << Puls2Bit)
#define BuzzVal		(1 << BuzzBit)
#define I2CslvCVal	(1 << I2CslvCBit)
#define I2CslvDVal	(1 << I2CslvDBit)

#define LCDLuceVal	(1 << LCDLuceBit)


#define TCREAD		0b11110001					 //TRIS register for SDA read
#define TCWRITED	0b01110001				   //TRIS register for SDA write
#define TCWRITEC	0b10110001				   //TRIS register for SCL write (wait)
#ifndef USA_232
#define PCINIT   0b00000000
#else
#define PCINIT   0b11000000
#endif

#define TCINIT		TCREAD				//Initial TRIS value (PORTC)
																// 1=input e cmq. per RS232! (anche per SPI hardware), 0=output per 2PWM!




#define Led0Val		(1 << Led0Bit)
#define Led1Val		(1 << Led1Bit)
#define Led2Val		(1 << Led2Bit)



#endif //IO_CFG_H
