/* LCD.c a 4 bit, come da KIT N.E. LX1525

*/

#include <p18cxxx.h>
#include "typedefs.h"
#include <stdlib.h>
#if defined(__18CXX)
#include <delays.h>
#endif
#include "frequenz.h"
#include "plib.h"
#include "io_cfg.h"
#include "lcd.h"

void subLCDInit(void);

void subLCDget(void);




void SetBeep(void);

#if defined(__18CXX)
#pragma udata
#endif

byte LCDCols;			
byte LCDRows;			
byte LCDX;				
byte LCDY;			  
byte LuceLCD;					// mirrored in EEPROM
byte ContrLCD;		
byte StatoLCD;					// usato per indicare se il cursore  attivo (b0);
byte BeepFreq;
byte TimerBuzz;

byte FLAGS_ESC;					// per sequenze ESCAPE: 0=SetLuce, 1=SetContr, 2=SetFreq; 3=SetLed; 4&5=SetXY (1&2); 6=SetKClick; 7=invert_bitmap
byte FLAGS;

word I2CAddr;
byte I2CBuffer[8];					// buffer 16 byte per ric./trasm. I2C-slave (BANCO 1)
byte I2CData;
byte I2CCnt;
byte I2CCntB;

byte lcd_temp0;			// usato dentro LCDOutCmd da 6963, in GetCharDef da KS0108
byte lcd_temp1;			// temporanea, usata da LCD
byte lcd_temp2;
byte lcd_temp3;
byte LCDtmpBuf[21];					// temp per scroll display (almeno 20 byte), BANCO 1


volatile byte kbScanCode;			// il num. progressivo del tasto scandito

volatile byte kbKeys[MAX_TASTI_COMTEMPORANEI];			// stato dei tasti		(max 4=MAX_TASTI_COMTEMPORANEI contemp.)
byte kbKeysOld[MAX_TASTI_COMTEMPORANEI];				// stato dei tasti old


#define LCD_DELAY { Nop(); 	Nop(); 	Nop();	ClrWdt(); }

byte LCDOutPort4Bit(byte );

void I2CDelay(void);

void KBInit(void);
void kbEmptyBuf(void);
byte checkKey(void);


// ---------------------------------------------------------------------


//lang_1:				// italiano (in caso si volessero + lingue...)

rom const char CopyrDate[] = {"29/11/17 - S/N 0001\n" };
//	messg "Numero di serie: SERNUM"

rom static const char Copyright[32] = { 'P','I','C','B','e','l','l',' ','f','w',' ','v',VERNUMH+'0','.',VERNUML/10+'0',(VERNUML % 10)+'0',0 };
//	dt	"ADPM Synthesis - Terminale grafico USB "
//	dt  VERNUMH+'0','.',VERNUML/10+'0',(VERNUML % 10)+'0', ' '
//	dt "08/04/05",0
rom const char *CopyrString=Copyright;
rom static const char *Copyr1= { "(C) ADPM Synthesis 1999-2017 - G.Dar\n" };

//	retlw  VERNUML
//	retlw  VERNUMH


rom const char String2_l1[] = {	'A','D','P','M','_','F','r','e','q',' ','v',VERNUMH+'0','.',VERNUML/10+'0',(VERNUML % 10)+'0',0 };



void aggLuceContrasto(void) {

//per ora no
	}


  



// ---------------------------------------------------------------------

void print_init(void) {

	LCDCls();
	LCDX= (LCD_MAX_COLS-16)/2;				// "ADPM_Freq vx.xx": 16 char
	LCDXY_();

	LCDWriteROM(String2_l1);

	}


// ---------------------------------------------------------------------
#define LCDPutBlank() LCDPutChar(' ')

void LCDPutChar(byte ch) { 


	if(FLAGS & (1 << IN_ESCAPE_CHAR))
		goto isInEscape;

	if(ch < ' ')
		goto NoPrint;


	if(ch & 0x80)			// i caratteri >=0x80 sono usati per i char user-defined...
		ch &= 0x7f;			// ...banalmente, se solo testo, li riporto al valore in LCD

LCDPutChar2:

  LCDOutData(ch);


  LCDX++;
  if(LCDX >= LCDCols)
		goto LCDPrintLF;
	if(LCDRows == 1) {
	  if(LCDX == 8)
			LCDXY_();
		}


NoPrint:

	switch(ch) {

		case 2:
			StatoLCD |= 1;
			LCDCursor(3);				// STX=cursor On
			break;

		case 3:
			StatoLCD &= 0xfe;		// ETX=cursor Off
			LCDCursor(0);
			break;

		case 7:
//  call StdBeep						; resa ASINCRONA (per velocita' I2C e anche SER)!
			SetBeep();
			break;
  

		case 9:
LCDtab:     
			LCDX++;

			LCDOutBlankRaw();		// meglio non LCDPutBlank, per lo stack!
							// cos per non gestisce l'a-capo...

		  if(LCDX & 7)
				goto LCDtab;
			break;

		case 11:
		  LCDScrollDown();
			break;

		case 12:					// FF = scroll up
  		LCDScrollUp();
			break;
	
		case 10:				// LF
LCDPrintLF:
			LCDX=0;
			LCDY++;
// METTERE un FLAG per disattivare scroll automatico (CHR$13) su ultima riga, per sfruttarla meglio??
		  LCDScroll();			// scroll SE necessario
			break;

		case 13:				// CR
		  LCDX=0;
			LCDXY_();
			break;
  

		case 8:					//; BS
		  if(LCDX) {				// solo se X>0...

			  LCDX--;

			  LCDOutCmd(16);
				LCDOutBlankRaw();
			  LCDOutCmd(16);
				}
			break;


		case 17:			// DC1..2 usati per Luce LCD
			LuceLCD=15;
LCDPC_NoPrint_17_:
			EEcopiaAEEPROM(&LuceLCD);
			aggLuceContrasto();			// occhio allo stack...!
			break;

		case 18:
			LuceLCD=0;
			goto LCDPC_NoPrint_17_;
			break;

		case 19:					// DC3..4 usati per Led on/off
			m_Led1Bit=1;
			break;

		case 20:
			m_Led1Bit=0;
			break;

		case 24:					// CAN (opp. FormFeed ?) = CLS
			LCDCls();
			break;



		case 27:					// ESC (comandi ausiliari)
			FLAGS |= (1 << IN_ESCAPE_CHAR);
			break;

		default:
// LCDPut1:			eliminato, eqv. a return
			break;

		}

isInEscape:
	if(FLAGS_ESC & 1)
		goto esc_SetLuce;
	if(FLAGS_ESC & 2)
		goto esc_SetContr;
	if(FLAGS_ESC & 4)
		goto esc_SetFreq;
	if(FLAGS_ESC & 8)
		goto esc_SetLed;
	if(FLAGS_ESC & 0x10)
		goto esc_SetXY;
	if(FLAGS_ESC & 0x20)
		goto esc_SetXY2;
	if(FLAGS_ESC & 0x40)
		goto esc_SetClick;
	goto isInEscape2;

esc_SetLuce:
	FLAGS_ESC=0;
	FLAGS &= ~(1 << IN_ESCAPE_CHAR);
	EEscrivi(&LuceLCD,ch-1);					// 1..16
	aggLuceContrasto();
	return;

esc_SetContr:
	FLAGS_ESC=0;
	FLAGS &= ~(1 << IN_ESCAPE_CHAR);
	EEscrivi(&ContrLCD,ch-1);					// 1..16
	aggLuceContrasto();
	return;

esc_SetFreq:
	FLAGS_ESC=0;
	FLAGS &= ~(1 << IN_ESCAPE_CHAR);
	// minimo 32 (v. PWM luce) (circa 10KHz per buzzer)
	BeepFreq= ch < 32 ? 32 : ch;
	aggLuceContrasto();
	return;

esc_SetLed:
	FLAGS_ESC=0;
	FLAGS &= ~(1 << IN_ESCAPE_CHAR);
//	LATB = ch;			//***** boh?
	return;

esc_SetXY:
	FLAGS_ESC=0;
	FLAGS_ESC |= (1 << 5);
	LCDY = ch;						// questo aspetta 2 parm...
	return;

esc_SetXY2:
	FLAGS_ESC=0;
	FLAGS &= ~(1 << IN_ESCAPE_CHAR);
	LCDX = ch;
	LCDXY_();
	return;

esc_SetClick:
	if(ch & 1)
		FLAGS &= ~(1 << NOKCLICK);
	else
		FLAGS |= (1 << NOKCLICK);
	return;

isInEscape2:
	switch(ch) {
		case 17:						// v. sopra
			FLAGS_ESC |= 1;
			break;
		case 18:						// un po' a caso...
			FLAGS_ESC |= 2;
			break;
		case 7:							// come "BEL"
			FLAGS_ESC |= 4;
			break;
		case 19:						// v. sopra
			FLAGS_ESC |= 8;
			break;
		case 91:						// vagamente ANSI... (VT100 dice: ESC [ ROW ; COLUMN f
			FLAGS_ESC |= 0x10;
			break;
		}

	if(!FLAGS_ESC)		// se non e' stato trovato nessun char. accettabile, tolgo QUI ESC!
		FLAGS &= ~(1 << IN_ESCAPE_CHAR);

	}


// ---------------------------------------------------------------------

void LCDXY_(void) {
	overlay byte i,lcd_temp2;

	lcd_temp2=0;

	switch(LCDRows) {
		case 1:
			if(LCDRows == 1 && LCDX >= 8) {			// patch x 1 riga... (v. anche PutChar)
				LCDOutCmd(0xc0 | (LCDX & 7));
				return;
				}
			else {
LCDXY1:
				i=LCDX & 0x7f;
				i+=lcd_temp2;
				LCDOutCmd(i | 0x80);
				}

			if(LCDX & 0x80) {
				LCDX &= 0x7f;
				LCDX-=LCDCols;
				}

			break;

		case 2:
LCDXY2:
			lcd_temp2=(LCDY & 1) * 0x40;
			goto LCDXY1;
			break;
  
		case 4:
			if(LCDCols < 40) {		// se e' 4x40 (M4024) va bene cosi' (hanno 2 ENABLE)...
				if(LCDY >= 2) {			// ...altrimenti (4x20), se la riga e' 2 opp. 3...
					LCDX+=LCDCols;		// ...aggiungo a X la larghezza (16 opp. 20)
					LCDX |= 0x80;			 //e me lo segno per poi ri-toglierla!
					}
				}
			goto LCDXY2;
			break;

		default:								// NON DEVE succedere!
			break;
			
		}
	}




// ---------------------------------------------------------------------


void subLCDInit(void) {

  LCDOutPort4Bit(0x3);        // init 1st
	Delay_mS(3);								// >4.1mS
  LCDOutPort4Bit(0x3);        // init 2nd
	Delay_mS(1);								// >100uS
  LCDOutPort4Bit(0x3);        // init 3rd
	Delay_mS(1);								// >100uS
  LCDOutPort4Bit(0x2);        // init 4th
	Delay_mS(1);								// >100uS
  LCDOutCmd(0b00101000);			      // function Set 0010NF**
  LCDOutCmd(0b00001100);      // display ON/OFF = 00001DCB, Display ON,Cursor&Blink OFF
	        // entry Mode Set = 000001IS, increment & shift off
  LCDOutCmd(0b00000110);	             // cursore, blink e ON/OFF
	}


const rom byte LCDUserDefChar[][8]={ 
	{ 0b00000110,0b00001001,0b00011100,0b00001000,0b00011100,0b00001001,0b00000110,0b00000000	}		// EURO
	};

void subLCDdefChar(void) {
	overlay byte x,y;

	LCDOutCmd(0x48);			// definisco char user-def partendo da 1 (non 0!)
	for(y=0; y<1; y++) {
		for(x=0; x<8; x++) {
		  LCDOutData(LCDUserDefChar[x][y]);
			}
		}
  
	}


byte LCDInit(void) {    

	LCDY=0;
	Delay_mS(10);				// 20mS
  subLCDInit();
	subLCDdefChar();

  LCDRows=LCD_MAX_ROWS;            // PARAMETRIZZARE (da FLASH?)
//            cp  3
//            jp m,LCDInit2 

//            movlw 2
//            movwf LCDY
//            call subLCDInit
            
LCDInit2:
  LCDCols=LCD_MAX_COLS;            // PARAMETRIZZARE (da FLASH?)
	LCDCls();
	}
// CONTINUA

// ---------------------------------------------------------------------

void LCDCls(void) {

//	movfw LCDRows
//  cp  3
//            jp m,LCDCls1
//            ld  a,2
//            ld  (LCDY),a
//            dec a
//  call LCDOutPort

LCDCls1:    
	LCDX=0;
  LCDY=0;
//	bcf  STATUS,C

  LCDOutCmd(1);       //                       ; cls & home (comando 1)

	Delay_mS(2);				// PERCHe' QUI NON ASPETTA BUSY!!

	}

// ---------------------------------------------------------------------

byte LCDOutPort(byte c, byte n) {												// scrivo W nel display
                                  // dato se C=1, comando se C=0

// qua non abbiamo "R/W" 
//	m_LCDRWBit=1;										// torno in lettura
//	m_LCDRSBit=0;										// preparo per leggere Flag
	lcd_temp1=0;


/* qua non abbiamo "R/W" ... uso un banale ritardo...
	while(1) {
//	clrwdt																; incorporato in LCD_DELAY
		m_LCDEnBit=1;										// scrivo!
		LCD_DELAY;
	            
		if(!(PORTB & 0x80))											// esco se b7=0
			goto LCDOut1;

		m_LCDEnBit=0;
		LCD_DELAY;

//	decf  lcd_temp1,f
//	bz  LCDDelay1_err
		}*/

	Delay_uS(100);

LCDDelay1_err:
LCDOut1:
//	m_LCDEnBit=0;							// rimetto cmq a posto EN...

	LCDOutPortNoW(c,n);
	}

byte LCDOutPortNoW(byte c,byte n) {									// qui per non aspettare BUSY!
// a 4 bit
	overlay byte i;

	m_LCDRSBit=c;										// C=1, dato		// C=0, comando

//	m_LCDRWBit=0;										// vado in scrittura
//	TRISB &= 0x0f;


	LCD_DELAY;
	m_LCDEnBit=1;										// scrivo!
	LCD_DELAY;
	            
	i=n & 0xf0;
	LATB=i | (LATB & 0xf);
	LCD_DELAY;

	m_LCDEnBit=0;
	LCD_DELAY;


	LCD_DELAY;
	m_LCDEnBit=1;										// scrivo!
	LCD_DELAY;
	            
	i=n << 4;
	LATB=i | (LATB & 0xf);
	LCD_DELAY;

	m_LCDEnBit=0;
	LCD_DELAY;

//	TRISB |= 0xf0;
//	m_LCDRWBit=1;										// torno in lettura

	m_LCDRSBit=1;
LCDOut2:
	
  return n;
	}

byte LCDOutPort4Bit(byte n) {

	m_LCDRSBit=0;										// C=0, comando
//	m_LCDRWBit=0;										// vado in scrittura
//	TRISB &= 0x0f;

	LCD_DELAY;
	m_LCDEnBit=1;										// scrivo!
	LCD_DELAY;
	      
	n <<= 4;
	LATB=n | (LATB & 0xf);
	LCD_DELAY;

	m_LCDEnBit=0;
	LCD_DELAY;

//	TRISB |= 0xf0;
//	m_LCDRWBit=1;										// torno in lettura

  return n;
	}


void LCDOutBlankRaw(void) {
  char c;

	c=' ';

  LCDOutData(c);
	}


// ---------------------------------------------------------------------

void LCDWriteROM(rom const char *s) {

	while(*s)
	  LCDPutChar(*s++);
	}

// ---------------------------------------------------------------------

void LCDWrite(const char *p) {
	
	while(*p)
	  LCDPutChar(*p++);
	}

void LCDWriteN(const char *p, byte n) {
	
	do {
	  LCDPutChar(*p++);
		} while(--n);
	}

// ---------------------------------------------------------------------
// in questo KIT, non c' READ (R/W) quindi niente da fare!
// in pi essendo a una sola riga... NON serve.

void subLCDget(void) {
	overlay byte *p;
	overlay byte i,lcd_temp1;

	p=LCDtmpBuf;

	for(i=0; i<LCDCols; i++) {

		m_LCDRSBit=0;										// preparo per leggere Flag
//		m_LCDRWBit=1;										// vado in lettura
		TRISB=0xff;

		lcd_temp1=0;
subLCDg3:   
		ClrWdt();
		m_LCDEnBit=1;										// strobe!
		LCD_DELAY
	            
		if(!(PORTB & 0x80))										// esco se b7=0
			goto subLCDg3_ok;

		m_LCDEnBit=0;
		LCD_DELAY

//	decf  lcd_temp1,f
//	bz  subLCDg3_err
		goto subLCDg3;

subLCDg3_err:
subLCDg3_ok:

		m_LCDRSBit=1;										// dato

		LCD_DELAY
		m_LCDEnBit=1;										// leggo!
		LCD_DELAY

		Nop();
		Nop();
		*p++=PORTB;
		Nop();
		Nop();
		Nop();

		m_LCDEnBit=0;
		LCD_DELAY
		}

	}


void LCDScroll(void) {
	overlay byte i,*p;
	overlay byte lcd_temp4;

	if(LCDRows > 1) {				// se 1 riga, niente scroll!

		if(LCDY == LCDRows) {

LCDScrollUp:

			for(lcd_temp4=0; lcd_temp4<(LCDRows-1); lcd_temp4++) {				// riga corrente
		
				lcd_temp4++;
				i= lcd_temp4 & 1 ? 0x40 : 0;				// calcolo posizione della riga source
				i+= lcd_temp4 & 2 ? LCDCols : 0;
				LCDOutCmd(i | 0x80);
			
				subLCDget();
		
				lcd_temp4--;
				i= lcd_temp4 & 1 ? 0x40 : 0;				// calcolo posizione della riga dest
				i+= lcd_temp4 & 2 ? LCDCols : 0;
				LCDOutCmd(i | 0x80);
		
				p=LCDtmpBuf;
				for(i=0; i<LCDCols; i++)
					LCDOutData(*p++);
		
			// se c'e' clock attivo su prima riga, limitare lo scroll?
				}
		
			LCDY--;
			LCDClearCurrentLine();
			}
	
	  LCDXY_();
		}

	}


void LCDScrollDown(void) {
	overlay byte i,*p;
	overlay byte lcd_temp4;

	lcd_temp4=LCDRows-1;					// riga corrente

	do {

		lcd_temp4--;
		i= lcd_temp4 & 1 ? 0x40 : 0;				// calcolo posizione della riga source
		i+= lcd_temp4 & 2 ? LCDCols : 0;
		LCDOutCmd(i | 0x80);
  
		subLCDget();

		lcd_temp4++;
		i= lcd_temp4 & 1 ? 0x40 : 0;				// calcolo posizione della riga dest
		i+= lcd_temp4 & 2 ? LCDCols : 0;
		LCDOutCmd(i | 0x80);

		p=LCDtmpBuf;

		for(i=0; i<LCDCols; i++)
			LCDOutData(*p++);

// se c'e' clock attivo su prima riga, limitare lo scroll?
		} while(--lcd_temp4);


	LCDY=0;
	LCDClearCurrentLine();


  LCDXY_();

	}



void LCDPrintHex(byte w) {					// stampa in HEX il byte in W
	overlay byte n;

	n=(w >> 4) & 0xf;
	if(n>=10)
		n+=7;
	LCDPutChar(n+'0');
	n=w & 0xf;
	if(n>=10)
		n+=7;
	LCDPutChar(n+'0');
	LCDPutBlank();
	}




void LCDPrintDec2(byte n) {				// stampa byte in W come decimale a 2 cifre ('0' trailing)

	}



void LCDClearLine(byte n) {					// entra n=linea da pulire
	overlay byte temp;
	
	LCDX=0;
	LCDXY(n);
	temp=LCDCols;
	while(temp--) {
		LCDOutBlankRaw();
		}
	LCDXY_();					// in caso servisse, sono a inizio riga!
	}



// ----------------------------------------------------------------------------

void KBInit(void) {

#if defined(__18CXX)
	TRISB |= 0x1;							// 
	TRISA |= 0xc;
#else
	TRISB |= 0b00000111;							// 
	TRISA |= 0b00000000;
#endif

	}


void kbEmptyBuf(void) {

	kbKeys[0]=0;
	kbKeys[1]=0;
	kbKeys[2]=0;
	kbKeys[3]=0;
	}

// ----------------------------------------------------------------------------

byte checkKey(void) {
	overlay byte *p;
	overlay byte n;
/*
	p=kbKeys;
	for(n=0; n<MAX_TASTI_COMTEMPORANEI; n++) {
		ClrWdt();
		if(!*p) {							//	cerco un posto vuoto x metterci lo scan code
			*p=kbScanCode;
			goto checkKey_fine;
			}
		p++;
		}

#ifdef USA_KB_BUZZER
// suonare male!
	PR2=BEEP_LOW_FREQ;

	CCPR1L=BEEP_LOW_FREQ/2;					// /2 (duty cycle 50%)
// il problema  che cambiando freq. qui, cambia anche quella del var. luce LCD...
// quindi occorre aggiornare quel valore CCPR2L

	n=250;
	while(n--) {
  	Delay10KTCYx(100);								 // ritardo almeno 20mSec
		ClrWdt();		// 1
		}

	// spengo PWM (non si muove mai...)
	CCPR1L=255;
#endif

	return 0;
	// non ho pi posto (max 4 tasti contemporanei!) dare errore!! (tutti 0x1)

checkKey_fine:
*/
	return 1;									// 
	}




byte GetKBchar(byte temp) {
/*
	switch(temp) {				// gli scan-code partono da 1...
		case 1:		return '0';		break;
		case 2:		return '1';		break;
		case 3:		return '2';		break;
		case 4:		return '3';		break;
		case 5:		return '4';		break;
		case 6:		return '5';		break;
		case 7:		return '6';		break;
		case 8:		return '7';		break;
		case 9:		return '8';		break;
		case 10:		return '9';		break;
		case 11:		return 'A';		break;
		case 12:		return 'B';		break;
		case 13:		return 'C';		break;
		case 14:		return 'D';		break;
		case 15:		return 'E';		break;
		case 16:		return 'F';		break;
		default:		return 0;		break;
	
		}
*/
	}







void SetBeep(void) {									// asincrono, viene poi resettato da interrupt

	PR2=BeepFreq;

	CCPR1L=BeepFreq >> 1;		// /2 (duty cycle 50%)
// il problema  che cambiando freq. qui, cambia anche quella del var. luce LCD...
// quindi occorre aggiornare quel valore CCPR2L
// Cio' viene fatto in aggLuceContrasto, che viene chiamata ANCHE quando si setta freq. buzz. con il medesimo cmd. I2C

//	movlw  00000111b
//	movwf  T2CON
	TimerBuzz=2;
	}


void drawLinea(byte x1,byte y1,byte x2,byte y2) {		// entrano coord. (lcd_temp2:lcd_temp3) (lcd_temp4:lcd_temp5)
								// le X VENGONO ALTERATE!
								//	alla fine si fa LCDXY per rimettere l'addr ptr a posto per il testo...
	byte temp;
	word bPos;
	byte lcd_temp5;


	}


void drawRect(byte x1,byte y1,byte x2,byte y2,byte w) {
		// entrano coord. (lcd_temp6:lcd_temp7) (lcd_temp8:lcd_temp9); W=0 se vuoto, 1 se pieno, 2=grigio



	}



/*------------------------------------------------------------------
;	Gestione EEProm esterna
;------------------------------------------------------------------*/



unsigned char readExtByte() {
	return;
	}

/**************************************************************
       Start Bit Subroutine
       this routine generates a start bit
       (Low going data line while clock is high)          
**************************************************************/

void DoI2CStart(void) {

#ifdef I2C_MSSP_HARDWARE			// fare?
#endif

//	SWStartI2C();
	m_I2CslvCBit=0;					// make sure clock is low
	m_I2CslvDBit=1;					// make sure data is high
	I2CDataTris=0;				// set data and clock lines for output
	I2CDelay();
	m_I2CslvCBit=1;					// set clock high
	I2CDelay();
	m_I2CslvDBit=0;					// data line goes low during high clock for start bit
	I2CDelay();
	m_I2CslvCBit=0;					// start clock train
	I2CDelay();
	}

/************************************************************
       Stop Bit Subroutine
       This routine generates a stop bit
       (High going data line while clock is high)
************************************************************/
void DoI2CStop(void) {

#ifdef I2C_MSSP_HARDWARE			// fare?
#endif

//	SWStopI2C();
	m_I2CslvDBit=0;						// make sure data line is low
	m_I2CslvCBit=0;						// make sure clock is low
	I2CDataTris=0;					// set data/clock lines as outputs
	m_I2CslvCBit=1;						// set clock high
	I2CDelay();
	m_I2CslvDBit=1;						// data goes high while clock high for stop bit
	I2CDelay();
	m_I2CslvCBit=0;						// set clock low again
	I2CDelay();
	I2CDataTris=1;					// set data line as input
	m_I2CslvCBit=1;						// set clock idle
	}

/*************************************************************
       BITOUT routine takes the bit of data in C and
       transmits it to the serial EE device
*************************************************************/
void I2CBITOUT(byte n) {

	I2CDataTris=0;					// set data,clock as outputs
	if(n)										// check for state of data bit to xmit
		m_I2CslvDBit=1;					// output a low bit     
	else
		m_I2CslvDBit=0;					// high? set data line high 
	Nop();										// dev'esserci un min. ritardo tra il cambio in DATA e la salita di CLK
	Nop();
	m_I2CslvCBit=1;							// set clock line high
	I2CDelay();
	m_I2CslvCBit=0;							// return clock line low
	I2CDelay();
//	retlw   0
	}
                                                             
/**************************************************************
       BITIN routine reads one bit of data from the 
       serial EE device and stores it in C
**************************************************************/
byte I2CBITIN(void) {
	overlay byte i;

	I2CDataTris=1;							// make sdata an input line
	I2CDelay();
	m_I2CslvCBit=1;							// set clock line high
	I2CDelay();		            // just sit here a sec
	if(m_I2CslvDBit)					  // read the data bit
		i=1;										// input bit was high
	else											// or...
		i=0;										// input bit was low
	m_I2CslvCBit=0;					    // set clock line low
	return i;
	}

/****************************************************************
       Transmit Data Subroutine
       This routine takes the byte of data stored in the
       'temp' register and transmits it to the serial EE device.
       It will then send 1 more clock to the serial EE for the
       acknowledge bit.  If the ack bit from the part was low
       then the transmission was sucessful.  If it is high, then
       the device did not send a proper ack bit and the ack
       fail LED will be turned on.
****************************************************************/
#define I2CTXSlaveAddrW() I2CTXByte(0xA0)
#define I2CTXSlaveAddrR() I2CTXByte(0xA1)


byte I2CTXByte(byte n) {

#ifdef I2C_MSSP_HARDWARE			// fare?
#endif

	overlay	byte I2CData,I2CCnt;
	overlay	byte t;

	I2CData=n;

	for(I2CCnt=0; I2CCnt<8; I2CCnt++) {				// set # bits to 8
		I2CBITOUT(I2CData & 0x80);		         // send the bit to serial EE
		I2CData <<= 1;							// rotate I2CData/txbuf left, bit in CARRY
		}												     // 8 bits done?
										        // no - go again
	t=I2CBITIN();								// read ack bit
	if(t)	{							// check ack bit
//		m_MemCardLedBit=1;				// spegne! set acknowledge fail LED if the device did not pull data low
		return  0;								// 0=ERR
		}

	return  1;								// 1=OK
	}
                    
/****************************************************************
       Receive data routine
       This routine reads one byte of data from the part
       into the 'I2CData' register.  It then sends a high 
       ACK bit to indicate that no more data is to be read
****************************************************************/
byte I2CRXByte(void) {

#ifdef I2C_MSSP_HARDWARE			// fare?
#endif

	overlay	byte I2CData,I2CCnt;
	overlay	byte t;

	I2CData=0;							// clear input buffer

	for(I2CCnt=0; I2CCnt<8; I2CCnt++) {				// set # bits to 8
		I2CData <<= 1;
		t=I2CBITIN();						// read a bit
		I2CData |= t;       // rotate I2CData 1 bit left
//		STATUSbits.C = 1;
//		Rlcf(input);            // Rotate the carry into the data byte
		}									    // 8 bits done?

													// no, do another
	return I2CData;
	}

/****************************************************************
       Receive data routine
       This routine reads one byte of data from the part
       into the 'I2CData' register.
****************************************************************/
byte I2CRX1Byte(void) {
	overlay byte I2CData,I2CCnt;
	overlay byte t;

//	SWReadI2C();

	I2CData=0;							// clear input buffer
	      
	for(I2CCnt=0; I2CCnt<8; I2CCnt++) {				// set # bits to 8
		I2CData <<= 1;
		t=I2CBITIN();						// read a bit
		I2CData |= t;       // rotate I2CData 1 bit left
													// no, do another
		} 						      // 8 bits done?
	// set ack bit = 1
	I2CBITOUT(1);						// to finish transmission

	return I2CData;
	}

/**************************************************************
       READ (read routine)
       This routine reads 8 consecutive addresses of the
       serial EE device starting at address 00 in the
       random access mode (non-sequential read). Reading 
       the device using the random access mode
       requires that the address pointer be set for every 
       byte before the read takes place. The address pointer
       is set by sending a 'write mode' control byte with the
       address of the location to read.  
***************************************************************/
byte I2CReadRandom(byte n) {
	overlay	byte I2CData;

//	bcf     port_a,ackf		; clear the ack fail LED if on
	
  DoI2CStart();						// generate start bit
												//
												// now send the write control byte and
												// address to set the pointer
												//
	I2CTXSlaveAddrW();		// get slave address and write mode
//	movfw   I2CAddr				; get word address
	I2CTXByte(n);		// and send it
											 
											 // now read one byte from the part
											 
	DoI2CStart();							// generate start bit
	I2CTXSlaveAddrR();			// get slave address and read mode
	I2CData=I2CRX1Byte();		// read 1 byte from serial EE
	DoI2CStop();							// send stop bit to end transmission
	
	return	I2CData;
	}

/**************************************************************
       READ (sequential read routine)

       This routine reads 8 consecutive addresses of the
       serial EE device starting at address 00 in the
       sequential read mode. Reading in this mode is more
       efficient than the random read mode as the control byte
       and address have to be sent only once at the beginning
       of the sequence.  As many consecutive addresses as
       needed can then be read from the part until a stop bit is
       sent.  In the read mode, the PIC 16C54 must send the acknowledge
       bit after every 8 data bits from the device.  When the
       last byte needed has been read, then the controller will
       send a high acknowledge bit and then a stop bit to halt
       transmission from the device. 
***************************************************************/
byte I2CRead8Seq(byte n,byte I2CCntB) {			// entra W=indirizzo part. lettura, FSR punta al buffer
	overlay	byte *p;

	//	bcf     port_a,ackf		; clear the ack fail LED if on
	p=I2CBuffer;						// bank1, ISP=0!
	
	DoI2CStart();						// generate start bit
	I2CTXSlaveAddrW();		// send slave address and write mode
													// get word address
	I2CTXByte(n);			// and send it        
	DoI2CStart();						// generate start bit
	I2CTXSlaveAddrR();		// send slave address and read mode

I2CRead8Seq2:
	*p++=I2CRXByte();				// read 1 byte from device
	if(!--I2CCntB) {					// are all 8 bytes read?
													// no, send low ack and do another
	// yes, send high ack bit 
		I2CBITOUT(1);					// to stop tranxmission
		DoI2CStop();						// and send a stop bit 
		return 8;
		}

	//  send low ack bit 
	I2CBITOUT(0);				// to continue transmission
	goto I2CRead8Seq2;		//and read another byte
	}

/**************************************************************
       READ16 (sequential read routine)

       This routine reads 8 consecutive addresses of the
       serial EE device starting at given 16bit address in the
       sequential read mode. Reading in this mode is more
       efficient than the random read mode as the control byte
       and address have to be sent only once at the beginning
       of the sequence.  As many consecutive addresses as
       needed can then be read from the part until a stop bit is
       sent.  In the read mode, the PIC 16C54 must send the acknowledge
       bit after every 8 data bits from the device.  When the
       last byte needed has been read, then the controller will
       send a high acknowledge bit and then a stop bit to halt
       transmission from the device. 
***************************************************************/
byte I2CRead16Seq(word n,byte I2CCntB) {			// entra indirizzo part. lettura, FSR punta al buffer
	overlay	byte *p;

//	bcf     port_a,ackf		; clear the ack fail LED if on
	
	DoI2CStart();						// generate start bit
	I2CTXSlaveAddrW();		// send slave address and write mode
				// move word address (HIGH)

	I2CTXByte(n >> 8);						// and send it        
				// move word address (LOW)
	I2CTXByte(n & 0xff);						// and send it        

	DoI2CStart();							// generate start bit
	I2CTXSlaveAddrR();			// send slave address and read mode

	p=I2CBuffer;						// bank1, ISP=0!
	
I2CRead16Seq2:
	*p++=I2CRXByte();				// read 1 byte from device
	if(!--I2CCntB) {					// are all 8 bytes read?
												// no, send low ack and do another
		// yes, send high ack bit 
		I2CBITOUT(1);					// to stop tranxmission
		DoI2CStop();						// and send a stop bit 
		return 16;
		}

  // send low ack bit 
	I2CBITOUT(0);		      // to continue transmission
	goto I2CRead16Seq2;   // and read another byte
	}

/****************************************************************
       Byte Write Routine
       This routine writes the data in "temp" to 
       8 consecutive bytes in the serial EE device starting
       at address 00.  This routine waits 10mS after every
       byte to give the device time to do the write.  This 
       program repeats forever. 
*****************************************************************/

void I2CWriteByte(byte n) {
	overlay	byte I2CAddr,I2CCntB,temp;

//	SWWriteI2C();

//	clrf    port_a				; clear all LEDs
	I2CAddr=n;						// set starting address to W
	temp=0x55;						// set data to write as 55h

	// set number of bytes
	I2CCntB=8;						// to write as to 8    

	do {
		DoI2CStart();						// generate start bit
		I2CTXSlaveAddrW();		// send slave address and write mode
		// move word address
		I2CTXByte(I2CAddr);		// and send it        
		I2CTXByte(temp);					// move data byte and transmit it
		DoI2CStop();						// generate stop bit

//	movlw   10
//	movwf   loops					; set delay time to give
//	call    WAIT					; 10 ms wait after every byte
		I2CWritePagePoll();

		I2CAddr++;						// add 1 to address counter
		} while(--I2CCntB);		// all 8 bytes written?

	}

/****************************************************************
       Page Write Routine

       This routine uses page mode to write the data in "temp" to 
       8 consecutive bytes in the serial EE device starting
       at address 00. This routine waits 10mS after every
       page to give the device time to do the write.  This
       routine executes forever 
*****************************************************************/

void I2CWritePage(byte n,byte I2CCntB) {					// entra W=indirizzo part. lettura, (FSR usato per buffer)
	overlay	byte *p;

//	clrf    port_a					; clear all LEDs
	p=I2CBuffer;					// bank1, ISP=0!

	DoI2CStart();							// generate start bit
	I2CTXSlaveAddrW();			// send slave address and write mode

	// move word address
	I2CTXByte(n);						// and send it        

I2CWritePage2:
	do {
		I2CTXByte(*p++);						// and transmit it
		} while(--I2CCntB);						// all n bytes written?
	DoI2CStop();							// yes,generate stop bit

//	movlw   10
//	movwf   loops						; set delay time to give
//	call    WAIT						; 10 ms wait after every byte
	Delay_S_(1);							// 66mS
	}

/****************************************************************
       Page Write 16 Routine (8 bytes, address a 16 bit)

       This routine uses page mode to write the data in "temp" to 
       8 consecutive bytes in the serial EE device starting
       at given address . This routine waits 10mS after every
       page to give the device time to do the write.  This
       routine executes forever 
*****************************************************************/
#define I2CWritePage16Default() I2CWritePage16(I2CAddr)


byte I2CWritePage16(word n,byte I2CCntB) {		// entra W=puntat. all'indirizzo part. lettura, (FSR usato per buffer)
	overlay	byte *p;

// clrf    port_a					; clear all LEDs


	DoI2CStart();							// generate start bit
	I2CTXSlaveAddrW();			// send slave address and write mode

							//	move word address (HIGH)
	I2CTXByte(n >> 8);						// and send it        
							// move word address (LOW)
	I2CTXByte(n & 0xff);						// and send it        

	p=I2CBuffer;						// bank1, ISP=0!

	do {
		I2CTXByte(*p++);				// and transmit it
		} while(--I2CCntB);					// move data byte 
	DoI2CStop();							// yes,generate stop bit

//	movlw   10
//	movwf   loops						; set delay time to give
//	call    WAIT						; 10 ms wait after every byte
	Delay_S_(1);						// 66mS
	return 16;
	}


byte I2CWritePagePoll(void) {
	byte I2Cpollcnt,i;

	I2Cpollcnt=100;						// set max number of times to poll as 100
	do {
		DoI2CStart();								// generate start bit
		i=I2CTXSlaveAddrW();				// send slave address and write mode
		if(i)										// was the ack bit low?
			goto exitpoll;				// yes, do another byte
		} while(--I2Cpollcnt);		 // is poll counter down to zero?
															 // no, poll again.  Otherwise the part is
//	bsf     port_a,timeout	; not responding in time so set timeout LED and continue on 

exitpoll:
		;
	}


void I2CDelay(void) {			// circa 2,5uSec (400KHz operation)
	overlay	unsigned char uSec=1;

	do {
		ClrWdt();							// Clear the WDT
		Delay1TCY();			// TARARE!! @40MHz??
		Delay1TCY();
		Delay1TCY();
//		Delay1TCY();
		} while(--uSec);
	}


//Delays W microseconds (includes movlw, call, and return) @ 24MHz
void Delay_uS_I2C(void) {

//	clrwdt			; 1
//	addlw	-1		; 1
//	nop					; 1
//	btfss	STATUS, Z		; 1 
//	goto	Delay_uS_I2C		; 2
	}


