#include <xc.h>
#include "plib.h"

// timers functions from C18:

/********************************************************************
*    Function Name:  OpenTimer0                                     *
*    Return Value:   void                                           *
*    Parameters:     config: bit definitions to configure Timer0    *
*    Description:    This routine resets the Timer0 regs to the     *
*                    POR state and configures the interrupt, clock  *
*                    source, edge and prescaler.                    *
*    Notes:          The bit definitions for config can be found    *
*                    in the timers.h file.                          *
********************************************************************/

void OpenTimer0(unsigned char config) {

//  T0CON = (0x7f & config);  // Configure timer, but don't start it yet
  TMR0 = 0;                // Reset Timer0 to 0x00
  INTCONbits.TMR0IF = 0;      // Clear Timer0 overflow flag

  if(config & 0x80)           // If interrupts enabled
    INTCONbits.TMR0IE = 1;    // Enable Timer0 overflow interrupt
  else
    INTCONbits.TMR0IE = 0;

//  T0CONbits.TMR0ON = 1;		// qua  sempre attivo
	}


void WriteTimer0(unsigned char timer0) {

  TMR0 = timer0;  // Write byte to Timer0
	}


unsigned int ReadTimer0(void) {

  return (TMR0);    // Return the int
	}


void CloseTimer0(void) {

//  T0CONbits.TMR0ON = 0;  // Disable Timer1
  INTCONbits.TMR0IE = 0;   // Disable Timer1 overflow interrupts
	}



void OpenTimer1(unsigned char config) {

  T1CON = (0x7e & config); // Set everything except 8/16 mode, and
                            // don't start timer yet

  TMR1H=0;             // Clear out timer registers
  TMR1L=0;
  PIR1bits.TMR1IF=0;

  if(config&0x80)      // Enable interrupts if selected
    PIE1bits.TMR1IE=1;
  else
    PIE1bits.TMR1IE=0;
  
  T1CONbits.TMR1ON = 1;  // Start Timer1
	}


void WriteTimer1(unsigned int timer1)
{
  union Timers timer;

  timer.lt = timer1;    // Save the 16-bit value in local

  TMR1H = timer.bt[1];  // Write high byte to Timer1 High byte
  TMR1L = timer.bt[0];  // Write low byte to Timer1 Low byte
}


unsigned int ReadTimer1(void)
{
  union Timers timer;

  timer.bt[0] = TMR1L;    // Read Lower byte
  timer.bt[1] = TMR1H;    // Read upper byte

  return (timer.lt);      // Return the 16-bit value
}


void CloseTimer1(void)
{
  T1CONbits.TMR1ON = 0;  // Disable Timer1
  PIE1bits.TMR1IE = 0;   // Disable Timer1 overflow interrupts 
}



// usart functions from C18:


