#define Nop() NOP()
#define ClrWdt() CLRWDT()


/* PIC18 timers peripheral library. */

/* used to hold 16-bit timer value */
union Timers {
  unsigned int lt;
  char bt[2];
};

#define TIMER_INT_OFF  0b01111111  //Disable TIMER Interrupt
#define TIMER_INT_ON   0b11111111  //Enable TIMER Interrupt

/* ***** TIMER0 ***** */
/* TIMER0 configuration masks -- to be 'anded' together and passed to the
 * 'open' routine. */
 
#define T0_16BIT       0b10111111  //Timer0 is configured as an 16-bit timer/counter
#define T0_8BIT        0b11111111  //Timer0 is configured as an 8-bit timer/counter

#define T0_SOURCE_INT  0b11011111  //Internal instruction cycle clock (CLKO) acts as source of clock
#define T0_SOURCE_INTOSC  0b11001111  //INTOSC
#define T0_SOURCE_EXT  0b11111111  //Transition on TxCKI pin acts as source of clock

#define T0_EDGE_RISE   0b11101111  //Increment on low-to-high transition on TxCKI pin
#define T0_EDGE_FALL   0b11111111  //Increment on high-to-low transition on TxCKI pin

#define T0_PS_1_1      0b11111111  //1:1 Prescale value (NO Prescaler)
#define T0_PS_1_2      0b11110000  //1:2 Prescale value
#define T0_PS_1_4      0b11110001  //1:4 Prescale value
#define T0_PS_1_8      0b11110010  //1:8 Prescale value
#define T0_PS_1_16     0b11110011  //1:16 Prescale value
#define T0_PS_1_32     0b11110100  //1:32 Prescale value
#define T0_PS_1_64     0b11110101  //1:64 Prescale value
#define T0_PS_1_128    0b11110110  //1:128 Prescale value
#define T0_PS_1_256    0b11110111  //1:256 Prescale value

void OpenTimer0 ( unsigned char config);
void CloseTimer0 (void);
unsigned int ReadTimer0 (void);
void WriteTimer0 ( unsigned char timer0);


/* ***** TIMER1 ***** */

/* TIMER1 configuration masks -- to be 'anded' together and passed to the
 * 'open' routine. */
 
#if defined(__18CXX)
#define T1_8BIT_RW       0b10111111  //Enables register read/write of Timer1 in two 8-bit operations
#define T1_16BIT_RW      0b11111111  //Enables register read/write of Timer1 in one 16-bit operation
#else
#define T1_SOURCE_CSC    0b11111111  //
#define T1_SOURCE_EXT    0b10111111  //
#define T1_SOURCE_INT    0b01111111  //
#define T1_SOURCE_INT4   0b00111111  //
#endif

#define T1_PS_1_1        0b11001111  // 1:1 prescale value
#define T1_PS_1_2        0b11011111  // 1:2 prescale value
#define T1_PS_1_4        0b11101111  // 1:4 prescale value
#define T1_PS_1_8        0b11111111  // 1:8 prescale value
#define T1_OSC1EN_OFF    0b11110111  // Timer 1 oscilator enable off
#define T1_OSC1EN_ON     0b11111111  // Timer 1 oscilator enable on
#define T1_SYNC_EXT_ON   0b11111011  // Synchronize external clock input
#define T1_SYNC_EXT_OFF  0b11111111  // Do not synchronize external clock input
#if defined(__18CXX)
#define T1_SOURCE_INT    0b11111101  //Internal instruction cycle clock (CLKO) acts as source of clock
#define T1_SOURCE_EXT    0b11111111  //Transition on TxCKI pin acts as source of clock
#else
#endif


void OpenTimer1 ( unsigned char config);
void CloseTimer1 (void);
unsigned int ReadTimer1 (void);
void WriteTimer1 ( unsigned int timer1);


#define EnablePullups()  { 	WPUA=0b11111111; 	WPUB=0b11111111; 	OPTION_REGbits.nWPUEN=0;}





